import pickle
import socket
import os
import signal
import errno
import re
from blueman.Constants import *
from blueman.Lib import create_bridge, destroy_bridge, BridgeException
from subprocess import call, Popen

class DnsMasqHandler(object):
	def __init__(self, netconf):
		self.pid = None
		self.netconf = netconf
		
	def do_apply(self):
		if not self.netconf.locked("dhcp") or self.netconf.ip4_changed:
			if self.netconf.ip4_changed:
				self.do_remove()
				
			if 1:
				rtr = "--dhcp-option=option:router,%s" % socket.inet_ntoa(self.netconf.ip4_address)
			else:
				rtr = "--dhcp-option=3 --dhcp-option=6" #no route and no dns
		
			start = self.netconf.ip4_address[:3] + chr(ord(self.netconf.ip4_address[3])+1)
			end = self.netconf.ip4_address[:3] + "\xfe" #.254
		
			args = "--pid-file=/var/run/dnsmasq.pan1.pid --bind-interfaces --dhcp-range=%s,%s,60m --except-interface=lo --interface=pan1 %s" % (socket.inet_ntoa(start), socket.inet_ntoa(end), rtr)
				
			argv = ["dnsmasq"] + args.split(" ")
			dprint(argv)
			p = Popen(argv)
		
			ret = p.wait()
		
			if ret == 0:
				dprint("Dnsmasq started correctly")
				f = open("/var/run/dnsmasq.pan1.pid", "r")
				self.pid = int(f.read())
				f.close()
				dprint("pid", self.pid)
				self.netconf.lock("dhcp")
			else:
				raise Exception("dnsmasq failed to start. Check the system log for errors")	
		
	
	def do_remove(self):
		if self.netconf.locked("dhcp"):
			os.kill(self.pid, signal.SIGTERM)
			self.netconf.unlock("dhcp")
			
class DhcpdHandler(object):
	def __init__(self, netconf):
		self.pid = None
		self.netconf = netconf
	
	def _read_dhcp_config(self):
		f = open(DHCP_CONFIG_FILE, "r")
		insection = False
		dhcp_config = ""
		existing_subnet = ""
		for line in f:
			if line == "#### BLUEMAN AUTOMAGIC SUBNET ####\n":
				insection = True
				
			if line == "#### END BLUEMAN AUTOMAGIC SUBNET ####\n":
				insection == False
				
			if not insection:
				dhcp_config += line
			else:
				existing_subnet += line
				
		f.close()
		
		return (dhcp_config, existing_subnet)
		
	def get_dns_servers(self):
		f = open("/etc/resolv.conf", "r")
		dns_servers = ""
		for line in f:
			server = re.search("^nameserver (.*)", line)
			if server:
				server = server.groups(1)[0]
				dns_servers += "%s, " % server;
			
		dns_servers = dns_servers.strip(", ")
		
		f.close()
		
		return dns_servers
		
	def _generate_subnet_config(self):
		dns = self.get_dns_servers()

		masked_ip = ""
		for i in range(4):
			masked_ip += chr(ord(self.netconf.ip4_address[i]) & ord(self.netconf.ip4_mask[i]))
		
		start = self.netconf.ip4_address[:3] + chr(ord(self.netconf.ip4_address[3])+1)
		end = self.netconf.ip4_address[:3] + "\xfe" #.254

		subnet = "#### BLUEMAN AUTOMAGIC SUBNET ####\n"
		subnet += "# Everything inside this section is destroyed after config change\n"
		subnet += """subnet %(ip_mask)s netmask %(netmask)s {
				option domain-name-servers %(dns)s;
				option subnet-mask %(netmask)s;
				option routers %(rtr)s;
				range %(start)s %(end)s;
				}\n""" % {"ip_mask": socket.inet_ntoa(masked_ip),
						  "netmask": socket.inet_ntoa(self.netconf.ip4_mask),
						  "dns": dns,
						  "rtr": socket.inet_ntoa(self.netconf.ip4_address),
						  "start": socket.inet_ntoa(start),
						  "end": socket.inet_ntoa(end)
						 }
		subnet += "#### END BLUEMAN AUTOMAGIC SUBNET ####\n"
		
		return subnet

	def do_apply(self):
		if not self.netconf.locked("dhcp") or self.netconf.ip4_changed:
			if self.netconf.ip4_changed:
				self.do_remove()
			
			dhcp_config, existing_subnet = self._read_dhcp_config()
		
			subnet = self._generate_subnet_config()
		
			#if subnet != self.existing_subnet:
			f = open(DHCP_CONFIG_FILE, "w")
			f.write(dhcp_config)
			f.write(subnet)
			f.close()
		
			p = Popen(["dhcpd3", "-pf", "/var/run/dhcp3-server/dhcpd.pan1.pid", "pan1"])
		
			ret = p.wait()
	
			if ret == 0:
				dprint("dhcpd started correctly")
				f = open("/var/run/dhcp3-server/dhcpd.pan1.pid", "r")
				self.pid = int(f.read())
				f.close()
				dprint("pid", self.pid)
				self.netconf.lock("dhcp")
			else:
				raise Exception("dhcpd failed to start. Check the system log for errors")		
		

 	def do_remove(self):
		dhcp_config, existing_subnet = self._read_dhcp_config()
		f = open(DHCP_CONFIG_FILE, "w")
		f.write(dhcp_config)
		f.close()
		
		if self.netconf.locked("dhcp"):
			os.kill(self.pid, signal.SIGTERM)
			self.netconf.unlock("dhcp")
		
		
class_id = 10

class NetConf(object):
	default_inst = None
	
	@classmethod
	def get_default(cls):
		if NetConf.default_inst:
			return NetConf.default_inst
			
		try:
			f = open("/var/lib/blueman/network.state", "r")
			obj = pickle.load(f)
			if obj.version != class_id:
				raise Exception
			NetConf.default_inst = obj
			f.close()	
			return obj
		except:
			n = cls()
			try:
				n.store()
			except:
				return n
			NetConf.default_inst = n
			return n
	
	def __init__(self):
		dprint()
		self.version = class_id
		self.dhcp_handler = None
		self.ipt_rules = []
		
		self.ip4_address = None
		self.ip4_mask = None
		self.ip4_changed = False

			
	def set_ipv4(self, ip, netmask):
		if self.ip4_address != ip or self.ip4_mask != netmask:
			self.ip4_changed = True
		
		self.ip4_address = ip
		self.ip4_mask = netmask
		
	def get_ipv4(self):
		return (self.ip4_address, self.ip4_mask)
		
	def enable_ip4_forwarding(self):
		f = open("/proc/sys/net/ipv4/ip_forward", "w")
		f.write("1")
		f.close()
		
		for d in os.listdir("/proc/sys/net/ipv4/conf"):
			f = open("/proc/sys/net/ipv4/conf/%s/forwarding" % d, "w")
			f.write("1")
			f.close()
		
	def add_ipt_rule(self, table, chain, rule):
		self.ipt_rules.append((table, chain, rule))
		args = ["/sbin/iptables", "-t", table, "-A", chain] + rule.split(" ")
		dprint(" ".join(args))
		ret = call(args)
		print "Return code", ret
		
	def del_ipt_rules(self):
		for table, chain, rule in self.ipt_rules:
			call(["/sbin/iptables", "-t", table, "-D", chain] + rule.split(" "))
		self.ipt_rules = []
		self.unlock("iptables")
		
	def set_dhcp_handler(self, handler):
		if not isinstance(self.dhcp_handler, handler):
			running = False
			if self.dhcp_handler:	
				self.dhcp_handler.do_remove()
				running = True
				
			self.dhcp_handler = handler(self)
			if running:
				self.dhcp_handler.do_apply()
	
	def get_dhcp_handler(self):
		if not self.dhcp_handler:
			return None
			
		return type(self.dhcp_handler)
		
	def apply_settings(self):
		if self.ip4_address == None or self.ip4_mask == None:
			if self.ip4_changed:
				self.ip4_changed = False
			self.store()
			return
		
		if self != NetConf.get_default():
			NetConf.get_default().remove_settings()
			NetConf.default_inst = self
		
		try:
			create_bridge("pan1")
		except BridgeException, e:
			if e.errno != errno.EEXIST:
				raise
		
		ip_str = socket.inet_ntoa(self.ip4_address)	
		mask_str = socket.inet_ntoa(self.ip4_mask)	
		
		if self.ip4_changed or not self.locked("ifconfig"):
			self.enable_ip4_forwarding()
			
			ret = call(["ifconfig", "pan1", ip_str, "netmask", mask_str, "up"])
			if ret != 0:
				raise Exception("Failed to setup interface pan1")
			self.lock("ifconfig")
			
		if self.ip4_changed or not self.locked("iptables"):
			self.del_ipt_rules()
		
			self.add_ipt_rule("nat", "POSTROUTING", "-s %s/%s -j MASQUERADE" % (ip_str, mask_str))
			self.add_ipt_rule("filter", "FORWARD", "-i pan1 -j ACCEPT")
			self.add_ipt_rule("filter", "FORWARD", "-o pan1 -j ACCEPT")
			self.add_ipt_rule("filter", "FORWARD", "-i pan1 -j ACCEPT")
			self.lock("iptables")
		
		if self.dhcp_handler:
			self.dhcp_handler.do_apply()
		self.ip4_changed = False
		
		self.store()
		
	def remove_settings(self):
		dprint(self)
		
		if self.dhcp_handler:
			self.dhcp_handler.do_remove()
					
		try:
			destroy_bridge("pan1")
		except:
			pass
		self.unlock("ifconfig")
		
		self.del_ipt_rules()
		
		self.store()
		
	def lock(self, key):
		f = open("/var/run/blueman-%s" % key, "w")
		f.close()
		
	def unlock(self, key):
		try:
			os.unlink("/var/run/blueman-%s" % key)
		except:
			pass
	
	def locked(self, key):
		return os.path.exists("/var/run/blueman-%s" % key)
	
	#save the instance of this class, requires root
	def store(self):
		if not os.path.exists("/var/lib/blueman"):
			os.mkdir("/var/lib/blueman")
		f = open("/var/lib/blueman/network.state", "w")
		pickle.dump(self, f, pickle.HIGHEST_PROTOCOL)
		f.close()
		




from BlueZInterface import BlueZInterface
from errors import raise_dbus_error
import types
import dbus


class PropertiesBlueZInterface(BlueZInterface):
    def __init__(self, interface, obj_path):
        super(PropertiesBlueZInterface, self).__init__(interface, obj_path)

        self._handler_wrappers = {}

        if self.__class__.get_interface_version()[0] >= 5 and obj_path:
            self.__properties_interface = dbus.Interface(self.get_dbus_proxy(), 'org.freedesktop.DBus.Properties')

    @raise_dbus_error
    def set(self, name, value):
        if type(value) is types.IntType:
            value = dbus.UInt32(value)
        if self.__class__.get_interface_version()[0] < 5:
            self.get_interface().SetProperty(name, value)
        else:
            return self.__properties_interface.Set(self.get_interface_name(), name, value)

    @raise_dbus_error
    def get_properties(self):
        if self.__class__.get_interface_version()[0] < 5:
            return self.get_interface().GetProperties()
        else:
            return self.__properties_interface.GetAll(self.get_interface_name())

    def _handle_signal(self, handler, signal, interface, obj_path, **kwargs):
        self.get_bus().add_signal_receiver(handler, signal, interface, 'org.bluez', obj_path, **kwargs)

    def _unhandle_signal(self, handler, signal, interface, obj_path, **kwargs):
        self.get_bus().remove_signal_receiver(
            handler, signal, self.get_interface_name(), 'org.bluez',
            self.get_object_path(), **kwargs
        )

    def handle_signal(self, handler, signal, **kwargs):
        if signal == 'PropertyChanged':
            if self.__class__.get_interface_version()[0] < 5:
                self._handle_signal(
                    handler, 'PropertyChanged', self.get_interface_name(), self.get_object_path(), **kwargs
                )
            else:
                def wrapper(interface_name, changed_properties, invalidated_properties, **kwargs):
                    if interface_name == self.get_interface_name():
                        for name, value in changed_properties.items():
                            handler(name, value, **kwargs)

                self._handler_wrappers[handler] = wrapper

                interface = 'org.freedesktop.DBus.Properties'

                self._handle_signal(wrapper, 'PropertiesChanged', interface, self.get_object_path(), **kwargs)
        else:
            raise Exception('Unknown signal: %s' % signal)

    def unhandle_signal(self, handler, signal, **kwargs):
        if signal == 'PropertyChanged':
            if self.__class__.get_interface_version()[0] < 5:
                self._unhandle_signal(handler, signal, self.get_interface_name(), self.get_object_path(), **kwargs)
            else:
                self._unhandle_signal(self._handler_wrappers[handler], 'PropertiesChanged',
                                      'org.freedesktop.DBus.Properties', self.get_object_path(), **kwargs)
        else:
            raise Exception('Unknown signal: %s' % signal)



# Copyright (C) 2009 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from blueman.iniparse.compat import SafeConfigParser

class BluezConfig(SafeConfigParser):
	def __init__(self, filename):
		SafeConfigParser.__init__(self)
		
		self.filename = filename
		
		self.path = "/etc/bluetooth/%s" % filename
		
		self.read(self.path)
		
	def write(self):
		f = open(self.path, "wb")
		SafeConfigParser.write(self, f)



# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from gi.repository import Gtk
from blueman.Constants import *
from blueman.plugins.ServicePlugin import ServicePlugin

from blueman.main.AppletService import AppletService
from blueman.main.Config import Config
from blueman.Functions import dprint


class Transfer(ServicePlugin):
	__plugin_info__ = (_("Transfer"), "gtk-open")
	def on_load(self, container):
		
		self.Builder = Gtk.Builder()
		self.Builder.set_translation_domain("blueman")
		self.Builder.add_from_file(UI_PATH +"/services-transfer.ui")
		self.widget = self.Builder.get_object("transfer")
		
		self.ignored_keys = []
		
		container.pack_start(self.widget, True, True, 0)
		a = AppletService()
		if "TransferService" in a.QueryPlugins():
			self.setup_transfer()
		else:
			self.widget.props.sensitive = False
			self.widget.props.tooltip_text = _("Applet's transfer service plugin is disabled")
		
		return True
		
	def on_enter(self):
		self.widget.props.visible = True
		
	def on_leave(self):
		self.widget.props.visible = False

	def on_property_changed(self, config, key, value):

		if key == "opp_enabled":
			self.Builder.get_object(key).props.active = value
		if key == "ftp_enabled":
			self.Builder.get_object(key).props.active = value
		if key == "ftp_allow_write":
			self.Builder.get_object(key).props.active = value
		if key == "shared_path":
			self.Builder.get_object(key).set_current_folder(value)
		if key == "browse_command":
			return
		if key == "shared_path":
			self.option_changed_notify(key, False)
		else:
			self.option_changed_notify(key)
	
	def on_apply(self):
		if self.on_query_apply_state() == True:

			try:
				a = AppletService()
			except:
				dprint("failed to connect to applet")
			else:
				c = self.get_options()
				if "opp_enabled" in c:
					if not self.TransConf.props.opp_enabled:
						a.TransferControl("opp", "destroy")
				
				if "ftp_enabled" in c:
					if not self.TransConf.props.ftp_enabled:
						a.TransferControl("ftp", "destroy")
				
				
				if "opp_accept" in c or "shared_path" in c or "opp_enabled" in c:
					if self.TransConf.props.opp_enabled:
						state = a.TransferStatus("opp")
						if state == 0: #destroyed
							a.TransferControl("opp", "create")
						elif state == 2: #running
							a.TransferControl("opp", "stop")
							a.TransferControl("opp", "start")
						elif state == 1:
							a.TransferControl("opp", "start")
							
				
				if "ftp_allow_write" in c or "shared_path" in c or "ftp_enabled" in c:
					if self.TransConf.props.ftp_enabled:
						state = a.TransferStatus("ftp")
						if state == 0: #destroyed
							a.TransferControl("ftp", "create")
						elif state == 2: #running
							a.TransferControl("ftp", "stop")
							a.TransferControl("ftp", "start")
						elif state == 1:
							a.TransferControl("ftp", "start")
				

				self.clear_options()
				
				
			dprint("transfer apply")


	
	def on_query_apply_state(self):
		opts = self.get_options()
		if opts == []:
			return False
		else:
			return True

			
	def setup_transfer(self):
		a = AppletService()
		status = a.TransferStatus("opp")
		if status == -1:
			self.widget.props.sensitive = False
			self.widget.props.tooltip_text = _("obex-data-server not available")
		
		self.TransConf = Config("transfer")
		self.TransConf.connect("property-changed", self.on_property_changed)
		opp_enabled = self.Builder.get_object("opp_enabled")
		ftp_enabled = self.Builder.get_object("ftp_enabled")
		ftp_allow_write = self.Builder.get_object("ftp_allow_write")
		opp_accept = self.Builder.get_object("opp_accept")
		shared_path = self.Builder.get_object("shared_path")
		obex_cmd = self.Builder.get_object("e_obex_cmd")
	
		opp_enabled.props.active = self.TransConf.props.opp_enabled
		ftp_enabled.props.active = self.TransConf.props.ftp_enabled
		ftp_allow_write.props.active = self.TransConf.props.ftp_allow_write
		opp_accept.props.active = self.TransConf.props.opp_accept
		if self.TransConf.props.browse_command == None:
			self.TransConf.props.browse_command = DEF_BROWSE_COMMAND
		
		obex_cmd.props.text = self.TransConf.props.browse_command
		
		if self.TransConf.props.shared_path != None:
			shared_path.set_current_folder(self.TransConf.props.shared_path)
		
		obex_cmd.connect("changed", lambda x: setattr(self.TransConf.props, "browse_command", x.props.text))
		opp_enabled.connect("toggled", lambda x: setattr(self.TransConf.props, "opp_enabled", x.props.active))
		ftp_enabled.connect("toggled", lambda x: setattr(self.TransConf.props, "ftp_enabled", x.props.active))
		ftp_allow_write.connect("toggled", lambda x: setattr(self.TransConf.props, "ftp_allow_write", x.props.active))
		opp_accept.connect("toggled", lambda x: setattr(self.TransConf.props, "opp_accept", x.props.active))
		shared_path.connect("current-folder-changed", lambda x: setattr(self.TransConf.props, "shared_path", x.get_filename()))




from PropertiesBlueZInterface import PropertiesBlueZInterface
from errors import raise_dbus_error


class NetworkServer(PropertiesBlueZInterface):
    @raise_dbus_error
    def __init__(self, obj_path):
        if self.__class__.get_interface_version()[0] < 5:
            interface = 'org.bluez.NetworkServer'
        else:
            interface = 'org.bluez.NetworkServer1'

        super(NetworkServer, self).__init__(interface, obj_path)

    @raise_dbus_error
    def register(self, uuid, bridge):
        self.get_interface().Register(uuid, bridge)

    @raise_dbus_error
    def unregister(self, uuid):
        self.get_interface().Unregister(uuid)



# coding=utf-8
# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from subprocess import *

from blueman.Constants import *
from blueman.Functions import *

from gi.repository import Gtk


def show_about_dialog(app_name, run=True):
	def uri_open(uri):
		Popen(['xdg-open', uri], stdout=PIPE)

	def email_hook(dialog, email, black_hole):
		uri_open('mailto:'+email)
		
	def url_hook(dialog, url, black_hole):
		uri_open(url)
	
	#FIXME find equivalents with introspection
	#Gtk.about_dialog_set_email_hook(email_hook, None)
	#Gtk.about_dialog_set_url_hook(url_hook, None)
	
	about = Gtk.AboutDialog()
	about.set_name(app_name)
	about.set_version(VERSION)
	about.set_translator_credits(_("translator-credits"))
	about.set_copyright('Copyright \xc2\xa9 2008 Valmantas Palikša\n'\
						'Copyright \xc2\xa9 2008 Tadas Dailyda')
	about.set_comments(_('Blueman is a GTK based Bluetooth manager'))
	about.set_website(WEBSITE)
	about.set_icon(get_icon('blueman'))
	about.set_logo(get_icon('blueman', 48))
	authors = ['Valmantas Palikša <walmis@balticum-tv.lt>',
				'Tadas Dailyda <tadas@dailyda.com>']
	about.set_authors(authors)
	if run:
		about.run()
		about.destroy()
	else:
		return about



import blueman.Lib as Lib
from gi.repository import GObject
from gi.repository import Gtk

from blueman.gui.Animation import Animation
from blueman.main.SpeedCalc import SpeedCalc
from blueman.Functions import get_icon, adapter_path_to_name
from blueman.Functions import format_bytes

import gettext

_ = gettext.gettext


class ManagerStats:
    def __init__(self, blueman):

        blueman.List.connect("adapter-changed", self.on_adapter_changed)

        if blueman.List.Adapter:
            self.hci = adapter_path_to_name(blueman.List.Adapter.get_object_path())
        else:
            self.hci = None

        self.time = None

        self.up_speed = SpeedCalc()
        self.down_speed = SpeedCalc()

        up = get_icon("blueman-up-inactive", 15)
        down = get_icon("blueman-down-inactive", 15)
        self.im_upload = Gtk.Image()
        self.im_upload.set_tooltip_text(_("Data activity indication"))
        self.im_upload.set_from_pixbuf(up)
        self.im_download = Gtk.Image()
        self.im_download.set_tooltip_text(_("Data activity indication"))
        self.im_download.set_from_pixbuf(down)
        self.im_upload.set_alignment(1, 0.5)
        self.im_download.set_alignment(1, 0.5)

        self.down_rate = Gtk.Label()
        self.down_rate.show()
        self.down_rate.set_alignment(1, 0.5)
        self.down_rate.set_tooltip_text(_("Total data received and rate of transmission"))

        self.up_rate = Gtk.Label()
        self.up_rate.show()
        self.up_rate.set_alignment(1, 0.5)
        self.up_rate.set_tooltip_text(_("Total data sent and rate of transmission"))

        self.uparrow = Gtk.Image()
        self.uparrow.set_tooltip_text(_("Total data sent and rate of transmission"))
        self.uparrow.set_from_stock(Gtk.STOCK_GO_UP, 1)
        self.uparrow.set_alignment(1, 0.5)

        self.downarrow = Gtk.Image()
        self.downarrow.set_tooltip_text(_("Total data received and rate of transmission"))
        self.downarrow.set_from_stock(Gtk.STOCK_GO_DOWN, 1)

        self.hbox = hbox = blueman.Builder.get_object("statusbar2")

        hbox.pack_start(self.uparrow, True, False, 0)
        hbox.pack_start(self.up_rate, False, False, 0)

        hbox.pack_start(self.downarrow, False, False, 0)
        hbox.pack_start(self.down_rate, False, False, 0)

        hbox.pack_start(Gtk.VSeparator(), False, False, 0)

        hbox.pack_start(self.im_upload, False, False, 0)
        hbox.pack_start(self.im_download, False, False, 0)
        hbox.show_all()
        self.on_adapter_changed(blueman.List, blueman.List.GetAdapterPath())

        self.up_blinker = Animation(self.im_upload,
                                    [get_icon("blueman-up-inactive", 15), get_icon("blueman-up-active", 15)])
        #self.down_blinker = Animation(self.im_download, ["/down_inactive.png", "/down_active.png"])
        self.down_blinker = Animation(self.im_download,
                                      [get_icon("blueman-down-inactive", 16), get_icon("blueman-down-active", 16)])

        self.start_update()

    def on_adapter_changed(self, List, adapter_path):
        if adapter_path != None:
            self.hci = adapter_path_to_name(adapter_path)
            self.hbox.props.sensitive = True
        else:
            self.hci = None
            self.hbox.props.sensitive = False

        self.up_speed.reset()
        self.down_speed.reset()

    def set_blinker_by_speed(self, blinker, speed):

        if speed > 0 and not blinker.status():
            blinker.start()

        if speed > (30 * 1024) and blinker.status():
            blinker.set_rate(20)
        elif speed > (20 * 1024) and blinker.status():
            blinker.set_rate(15)
        elif speed > (10 * 1024) and blinker.status():
            blinker.set_rate(10)
        elif speed > 1024 and blinker.status():
            blinker.set_rate(5)
        elif speed == 0 and blinker.status():

            blinker.stop()
        else:
            blinker.set_rate(1)


    def _update(self):
        #if self.hbox.parent.parent.parent.props.visible:

        if self.hci != None:
            devinfo = Lib.device_info(self.hci)
            _tx = devinfo["stat"]["byte_tx"]
            _rx = devinfo["stat"]["byte_rx"]

            tx, s_tx = format_bytes(_tx)
            rx, s_rx = format_bytes(_rx)

            _u_speed = self.up_speed.calc(_tx)
            _d_speed = self.down_speed.calc(_rx)

            self.set_blinker_by_speed(self.up_blinker, _u_speed)
            self.set_blinker_by_speed(self.down_blinker, _d_speed)

            u_speed, s_u_speed = format_bytes(_u_speed)
            d_speed, s_d_speed = format_bytes(_d_speed)

            self.set_data(tx, s_tx, rx, s_rx, u_speed, s_u_speed, d_speed, s_d_speed)

        return 1


    def start_update(self):
        self._update()
        self.timer = GObject.timeout_add(1000, self._update)

    def stop_update(self):
        if self.timer:
            GObject.source_remove(self.timer)

    def set_data(self, uploaded, u_name, downloaded, d_name, u_speed, us_name, d_speed, ds_name):
        self.down_rate.set_markup(
            '<span size="small">%.2f %s <i>%5.2f %s/s</i></span>' % (downloaded, d_name, d_speed, ds_name))
        self.up_rate.set_markup(
            '<span size="small">%.2f %s <i>%5.2f %s/s</i></span>' % (uploaded, u_name, u_speed, us_name))



from blueman.Functions import *
from blueman.plugins.AppletPlugin import AppletPlugin

from gi.repository import GObject
from gi.repository import Gtk


class Menu(AppletPlugin):
    __depends__ = ["StatusIcon"]
    __description__ = _("Provides a menu for the applet and an API for other plugins to manipulate it")
    __icon__ = "menu-editor"
    __author__ = "Walmis"
    __unloadable__ = False

    def on_load(self, applet):
        self.Applet = applet

        self.Applet.Plugins.StatusIcon.connect("popup-menu", self.on_popup_menu)

        self.__plugins_loaded = False

        self.__menuitems = []
        self.__menu = Gtk.Menu()

    def on_popup_menu(self, status_icon, button, activate_time):
        self.__menu.popup(None, None, Gtk.StatusIcon.position_menu,
                          status_icon, button, activate_time)

    def __sort(self):
        self.__menuitems.sort(lambda a, b: cmp(a[0], b[0]))

    def __clear(self):
        def each(child, _):
            self.__menu.remove(child)

        self.__menu.foreach(each, None)

    def __load_items(self):
        for item in self.__menuitems:
            self.__menu.append(item[1])
            if item[2]:
                item[1].show()

    def Register(self, owner, item, priority, show=True):
        self.__menuitems.append((priority, item, show, owner))
        if self.__plugins_loaded:
            self.__sort()
            self.__clear()
            self.__load_items()


    def Unregister(self, owner):
        for i in reversed(self.__menuitems):
            priority, item, show, orig_owner = i
            if orig_owner == owner:
                self.__menu.remove(item)
                self.__menuitems.remove(i)

    def on_plugins_loaded(self):
        self.__plugins_loaded = True
        self.__sort()
        self.__load_items()

    def get_menu(self):
        return self.__menu



from blueman.Functions import *
import dbus
from gi.repository import Gtk
from gi.repository import GObject
from blueman.plugins.AppletPlugin import AppletPlugin
from blueman.main.SignalTracker import SignalTracker
import blueman.bluez as bluez


class ShowConnected(AppletPlugin):
    __author__ = "Walmis"
    __depends__ = ["StatusIcon"]
    __icon__ = "blueman-active"
    __description__ = _(
        "Adds an indication on the status icon when Bluetooth is active and shows the number of connections in the tooltip.")

    def on_load(self, applet):
        self.num_connections = 0
        self.active = False
        self.initialized = False

        self.signals = SignalTracker()
        self.signals.Handle('bluez', bluez.Device(), self.on_device_property_changed, 'PropertyChanged')

    def on_unload(self):
        self.signals.DisconnectAll()
        self.Applet.Plugins.StatusIcon.SetTextLine(1, None)
        self.num_connections = 0
        self.Applet.Plugins.StatusIcon.IconShouldChange()


    def on_status_icon_query_icon(self):
        if self.num_connections > 0:
            self.active = True
            #			x_size = int(pixbuf.props.height)
            #			x = get_icon("blueman-txrx", x_size)
            #			pixbuf = composite_icon(pixbuf,
            #				[(x, pixbuf.props.height - x_size, 0, 255)])
            #
            #			return pixbuf
            return ("blueman-active",)
        else:
            self.active = False

    def enumerate_connections(self):
        self.num_connections = 0
        adapters = self.Applet.Manager.list_adapters()
        for adapter in adapters:
            devices = adapter.list_devices()
            for device in devices:
                props = device.get_properties()
                if "Connected" in props:
                    if props["Connected"]:
                        self.num_connections += 1

        dprint("Found %d existing connections" % self.num_connections)
        if (self.num_connections > 0 and not self.active) or \
                (self.num_connections == 0 and self.active):
            self.Applet.Plugins.StatusIcon.IconShouldChange()

        self.update_statusicon()

    def update_statusicon(self):
        if self.num_connections > 0:
            self.Applet.Plugins.StatusIcon.SetTextLine(0,
                                                       _("Bluetooth Active"))
            self.Applet.Plugins.StatusIcon.SetTextLine(1,
                                                       ngettext("<b>%d Active Connection</b>",
                                                                "<b>%d Active Connections</b>",
                                                                self.num_connections) % self.num_connections)
        else:
            self.Applet.Plugins.StatusIcon.SetTextLine(1, None)
            try:
                if self.Applet.Plugins.PowerManager.GetBluetoothStatus():
                    self.Applet.Plugins.StatusIcon.SetTextLine(0,
                                                               _("Bluetooth Enabled"))
            except:
                #bluetooth should be always enabled if powermanager is
                #not loaded
                self.Applet.Plugins.StatusIcon.SetTextLine(0,
                                                           _("Bluetooth Enabled"))

    def on_manager_state_changed(self, state):
        if state:
            if not self.initialized:
                GObject.timeout_add(0, self.enumerate_connections)
                self.initialized = True
            else:
                GObject.timeout_add(1000,
                                    self.enumerate_connections)
        else:
            self.num_connections = 0
            self.update_statusicon()

    def on_device_property_changed(self, key, value):
        if key == "Connected":
            if value:
                self.num_connections += 1
            else:
                self.num_connections -= 1

            if (self.num_connections > 0 and not self.active) or \
                    (self.num_connections == 0 and self.active):
                self.Applet.Plugins.StatusIcon.IconShouldChange()

            self.update_statusicon()

    def on_adapter_added(self, adapter):
        self.enumerate_connections()

    def on_adapter_removed(self, adapter):
        self.enumerate_connections()



# Copyright (C) 2009 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
import socket
import fcntl
from gi.repository import GObject
import struct
import subprocess
from blueman.Lib import get_net_address


class DhcpClient(GObject.GObject):
	__gsignals__ = {
		#arg: interface name eg. ppp0
		'connected' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT,)),
		'error-occurred' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT,)),
	}	
	
	quering = []
	
	def __init__(self, interface, timeout=30):
		GObject.GObject.__init__(self)
	
		self.interface = interface
		self.timeout = timeout
		
		self.DHCP_CLIENT = False
		try:
			self.dhclient = subprocess.Popen(["dhclient", "--version"])
			self.dhclient.terminate()
			self.DHCP_CLIENT = "DHCLIENT"
		except OSError:
			try:
				self.dhcpcd = subprocess.Popen(["dhcpcd", "--version"])
				self.dhcpcd.terminate()
				self.DHCP_CLIENT = "DHCPCD"
			except OSError:
				self.DHCP_CLIENT = "NONE"
	
	def Connect(self):
		if self.DHCP_CLIENT == "NONE":
			raise Exception("no DHCP client found, please install dhcpcd or dhclient")

		if self.interface in DhcpClient.quering:
			raise Exception("DHCP already running on this interface")
		else:
			DhcpClient.quering.append(self.interface)

		if self.DHCP_CLIENT == "DHCLIENT":
			try:	
				self.dhclient = subprocess.Popen(["dhclient", "-e", "IF_METRIC=100", "-1", self.interface])
			except:
				#Should never happen now...
				raise Exception("dhclient binary not found")
			GObject.timeout_add(1000, self.check_dhclient)
		elif self.DHCP_CLIENT == "DHCPCD":
			try:	
				self.dhcpcd = subprocess.Popen(["dhcpcd", "-m", "100", self.interface])
			except: 
				#Should never happen now...
				raise Exception("dhcpcd binary not found")		
			GObject.timeout_add(1000, self.check_dhcpcd)

		GObject.timeout_add(self.timeout*1000, self.on_timeout)

		
	def on_timeout(self):
		if self.DHCP_CLIENT == "DHCLIENT":
			if self.dhclient.poll() == None:
				dprint("Timeout reached, terminating dhclient")
				self.dhclient.terminate()
		elif self.DHCP_CLIENT == "DHCPCD":
			if self.dhcpcd.poll() == None:
				dprint("Timeout reached, terminating dhcpcd")
				self.dhcpcd.terminate()
		
	def check_dhclient(self):
		status = self.dhclient.poll()
		if status != None:
			if status == 0:
				def complete():
					ip = get_net_address(self.interface)
					dprint("bound to", ip)
					self.emit("connected", ip)	
				
				GObject.timeout_add(1000, complete)
				DhcpClient.quering.remove(self.interface)
				return False
				
			else:
				dprint("dhcp client failed with status code", status)
				self.emit("error-occurred", status)
				DhcpClient.quering.remove(self.interface)
				return False
			
		return True	
	
	def check_dhcpcd(self):
		status = self.dhcpcd.poll()
		if status != None:
			if status == 0:
				def complete():
					ip = get_net_address(self.interface)
					dprint("bound to", ip)
					self.emit("connected", ip)	
				
				GObject.timeout_add(1000, complete)
				DhcpClient.quering.remove(self.interface)
				return False
				
			else:
				dprint("dhcpcd failed with status code", status)
				self.emit("error-occurred", status)
				DhcpClient.quering.remove(self.interface)
				return False
			
		return True		



from blueman.Functions import *
from blueman.plugins.AppletPlugin import AppletPlugin
import blueman.bluez as Bluez
from blueman.bluez.errors import BluezDBusException
from blueman.main.SignalTracker import SignalTracker
import types


class PowerManager(AppletPlugin):
    __depends__ = ["StatusIcon", "Menu"]
    __unloadable__ = True
    __description__ = _("Controls Bluetooth adapter power states")
    __author__ = "Walmis"
    __icon__ = "gnome-power-manager"

    def on_load(self, applet):
        AppletPlugin.add_method(self.on_power_state_query)
        AppletPlugin.add_method(self.on_power_state_change_requested)
        AppletPlugin.add_method(self.on_power_state_changed)

        self.add_dbus_method(self.SetBluetoothStatus,
                             in_signature="b",
                             out_signature="")

        self.add_dbus_method(self.GetBluetoothStatus,
                             in_signature="",
                             out_signature="b")

        self.BluetoothStatusChanged = self.add_dbus_signal(
            "BluetoothStatusChanged",
            signature="b")

        self.Applet = applet

        self.item = create_menuitem(_("<b>Bluetooth Off</b>"), get_icon("gtk-stop", 16))
        self.item.get_child().set_markup(_("<b>Turn Bluetooth Off</b>"))

        self.item.props.tooltip_text = _("Turn off all adapters")

        self.signals = SignalTracker()
        self.signals.Handle('bluez', Bluez.Adapter(), self.adapter_property_changed, "PropertyChanged",
                            path_keyword="path")

        self.signals.Handle(self.item, "activate", lambda x: self.on_bluetooth_toggled())

        self.Applet.Plugins.Menu.Register(self, self.item, 0)

        self.adapter_state = True
        self.current_state = True
        self.power_changeable = True

        self.request_in_progress = False

        self.STATE_ON = 2
        self.STATE_OFF = 1
        self.STATE_OFF_FORCED = 0

    def on_unload(self):
        self.signals.DisconnectAll()
        self.Applet.Plugins.Menu.Unregister(self)

    @property
    def CurrentState(self):
        return self.current_state

    def on_manager_state_changed(self, state):
        if state:
            def timeout():
                self.adapter_state = self.get_adapter_state()
                self.RequestPowerState(self.adapter_state)

            GObject.timeout_add(1000, timeout)

    def get_adapter_state(self):
        adapters = self.Applet.Manager.list_adapters()
        for adapter in adapters:
            props = adapter.get_properties()
            if not props["Powered"]:
                return False
        return bool(adapters)

    def set_adapter_state(self, state):
        try:
            dprint(state)
            adapters = self.Applet.Manager.list_adapters()
            for adapter in adapters:
                adapter.set("Powered", state)

            self.adapter_state = state
        except Exception as e:
            dprint("Exception occurred", e)


    class Callback(object):
        def __init__(self, parent, state):
            self.parent = parent
            self.num_cb = 0
            self.called = 0
            self.state = state
            self.success = False
            self.timer = GObject.timeout_add(5000, self.timeout)

        def __call__(self, result):
            self.called += 1

            if result:
                self.success = True

            self.check()

        def check(self):
            if self.called == self.num_cb:
                dprint("callbacks done")
                self.parent.set_adapter_state(self.state)
                GObject.source_remove(self.timer)
                self.parent.request_in_progress = False

        def timeout(self):
            dprint("Timeout reached while setting power state")
            self.parent.UpdatePowerState()
            self.parent.request_in_progress = False

    def RequestPowerState(self, state):
        if self.current_state != state:
            if not self.request_in_progress:
                self.request_in_progress = True
                dprint("Requesting", state)
                cb = PowerManager.Callback(self, state)

                rets = self.Applet.Plugins.Run("on_power_state_change_requested", self, state, cb)
                cb.num_cb = len(rets)
                cb.check()
                self.UpdatePowerState()
            else:
                dprint("Another request in progress")

    def on_power_state_change_requested(self, pm, state, cb):
        cb(None)

    def on_power_state_query(self, pm):
        if self.adapter_state:
            return self.STATE_ON
        else:
            return self.STATE_OFF

    def on_power_state_changed(self, manager, state):
        pass


    #queries other plugins to determine the current power state
    def UpdatePowerState(self):
        rets = self.Applet.Plugins.Run("on_power_state_query", self)

        off = True in map(lambda x: x < self.STATE_ON, rets)
        foff = self.STATE_OFF_FORCED in rets
        on = self.STATE_ON in rets

        new_state = True
        if foff or off:

            self.item.get_child().set_markup(_("<b>Turn Bluetooth On</b>"))
            self.item.props.tooltip_text = _("Turn on all adapters")
            self.item.set_image(Gtk.Image.new_from_pixbuf(get_icon("gtk-yes", 16)))

            if foff:
                self.item.props.sensitive = False
            else:
                self.item.props.sensitive = True

            new_state = False

        elif on and self.current_state != True:
            self.item.get_child().set_markup(_("<b>Turn Bluetooth Off</b>"))
            self.item.props.tooltip_text = _("Turn off all adapters")
            self.item.set_image(Gtk.Image.new_from_pixbuf(get_icon("gtk-stop", 16)))
            self.item.props.sensitive = True

            new_state = True

        dprint("off", off, "\nfoff", foff, "\non", on, "\ncurrent state", self.current_state, "\nnew state", new_state)

        if self.current_state != new_state:
            dprint("Signalling", new_state)
            self.current_state = new_state

            self.BluetoothStatusChanged(new_state)
            self.Applet.Plugins.Run("on_power_state_changed", self, new_state)
            self.Applet.Plugins.StatusIcon.IconShouldChange()

    #dbus method
    def SetBluetoothStatus(self, status):
        self.RequestPowerState(status)

    #dbus method
    def GetBluetoothStatus(self):
        return self.CurrentState

    def adapter_property_changed(self, key, value, path):
        if key == "Powered":
            if value and not self.CurrentState:
                dprint("adapter powered on while in off state, turning bluetooth on")
                self.RequestPowerState(True)

            self.UpdatePowerState()

    def on_bluetooth_toggled(self):
        self.RequestPowerState(not self.CurrentState)

    def on_status_icon_query_icon(self):
        #opacity = 255 if self.GetBluetoothStatus() else 100
        #pixbuf = opacify_pixbuf(pixbuf, opacity)

        #if opacity < 255:
        #	x_size = int(pixbuf.props.height)
        #	x = get_icon("blueman-x", x_size)
        #	pixbuf = composite_icon(pixbuf, [(x, pixbuf.props.height - x_size, pixbuf.props.height - x_size, 255)])

        #return pixbuf
        if not self.GetBluetoothStatus():
            return ("blueman-disabled", "blueman-disabled")

    def on_adapter_added(self, path):
        adapter = Bluez.Adapter(path)

        def on_ready():
            if not self.adapter_state:
                adapter.set("Powered", False)
            else:
                adapter.set("Powered", True)

        wait_for_adapter(adapter, on_ready)




# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import Pango
from gi.repository import GObject
from blueman.gui.GtkAnimation import WidgetFade
from blueman.Constants import *
from blueman.Functions import get_icon

class MessageArea(Gtk.EventBox):
	_inst_ = None
	
	def __new__(cls):
		if not MessageArea._inst_:
			MessageArea._inst_ = super(MessageArea, cls).__new__(cls)
		
		return MessageArea._inst_

	def __init__(self):
		GObject.GObject.__init__(self)
		
		self.hbox = Gtk.HBox()
		self.hbox.show()
		
		self.text = ""
		
		self.set_app_paintable(True)
		
		self.anim = WidgetFade(self.hbox, self.hbox.get_style().lookup_color("base_color")[1])
		self.hl_anim = WidgetFade(self.hbox, Gdk.Color(65535,0,0))
		
		self.setting_style = False
		
		self.hbox.props.spacing = 4
		self.hbox.set_border_width(2)
		
		self.icon = Gtk.Image()
		self.icon.props.xpad = 4
		self.label = Gtk.Label()
		self.label.props.xalign = 0
		self.label.set_ellipsize(Pango.EllipsizeMode.END)
		self.label.set_single_line_mode(True)
		self.label.set_selectable(True)
		
		
		self.b_more = Gtk.Button(_("More"))
		im = Gtk.Image()
		im.set_from_stock(Gtk.STOCK_DIALOG_INFO, Gtk.IconSize.MENU)
		im.show()
		self.b_more.set_image(im)		
		self.b_more.props.relief = Gtk.ReliefStyle.NONE
		
		im = Gtk.Image()
		im.set_from_stock(Gtk.STOCK_CANCEL, Gtk.IconSize.MENU)
		im.show()
		self.b_close = Gtk.Button()
		self.b_close.add(im)
		self.b_close.props.relief = Gtk.ReliefStyle.NONE
		self.b_close.props.tooltip_text = _("Close")
		
		self.hbox.pack_start(self.icon, False, False, 0)
		self.hbox.pack_start(self.label, True, False, 0)
		self.hbox.pack_start(self.b_more, False, False, 0)
		self.hbox.pack_start(self.b_close, False, False, 0)
		
		self.add(self.hbox)
		
		self.icon.show()
		self.b_close.show()
		self.label.show()
		self.b_more.show()
		
		self.b_close.connect("clicked", self.on_close)
		self.b_more.connect("clicked", self.on_more)

		self.hbox.connect("draw", self.draw)
		self.b_close.connect("style-set", self.style_set)
		
	def on_more(self, button):
		d = Gtk.MessageDialog(parent=None, flags=0, type=Gtk.MessageType.INFO, buttons=Gtk.ButtonsType.CLOSE)
		
		d.props.text = self.text
		
		d.run()
		d.destroy()
		
	def style_set(self, widget, prev_style):
		if self.setting_style:
			return
		
		#This is a hack needed to use the tooltip background color
		window = Gtk.Window(Gtk.WindowType.POPUP)
		window.set_name("gtk-tooltip")
		window.ensure_style()
		style = window.get_style()
		window.destroy()
		
		self.setting_style = True
		
		#recursively set style
		def _set_style(wg):
			if isinstance(wg, Gtk.Container):
				for w in wg:
					if not isinstance(w, Gtk.Button):
						_set_style(w)

			wg.set_style(style)
		
		_set_style(self)
		self.anim.color = self.hbox.get_style_context().get_background_color(0)
		self.queue_draw()
		
		self.setting_style = False
		
		
	def on_close(self, button):
		def on_finished(anim):
			anim.disconnect(sig)
			self.props.visible = False
			anim.freeze()
		
		sig = self.anim.connect("animation-finished", on_finished)
		self.anim.thaw()
		self.anim.animate(start=1.0, end=0.0, duration=500)

	@staticmethod
	def close():
		MessageArea._inst_.on_close(None)
	
	@staticmethod
	def show_message(*args):
		MessageArea._inst_._show_message(*args)
	
	def _show_message(self, text, icon=Gtk.STOCK_DIALOG_WARNING):
		self.text = text
		
		self.label.set_tooltip_text(text)
		self.icon.set_from_stock(icon, Gtk.IconSize.MENU)
		
		if icon == Gtk.STOCK_DIALOG_WARNING:
			self.hl_anim.color = Gdk.Color(65535,0,0)
		else:
			self.hl_anim.color = Gdk.Color(0,0,65535)
		
		def on_finished(anim):
			anim.disconnect(sig)
			anim.freeze()		
		
		if not self.props.visible:
			sig = self.anim.connect("animation-finished", on_finished)
			
			self.anim.thaw()
			
			self.show()
			self.anim.animate(start=0.0, end=1.0, duration=500)
		else:
			sig = self.hl_anim.connect("animation-finished", on_finished)
			self.hl_anim.thaw()
			self.hl_anim.animate(start=0.7, end=1.0, duration=1000)
		
		lines = text.split("\n")
		if len(lines) > 1:
			self.label.props.label = lines[0] + "..."
			self.b_more.props.visible = True
		else:
			self.label.props.label = text
			self.b_more.props.visible = False
		
	def draw(self, window, cr):
		rect = window.get_allocation()
		Gtk.paint_box(window.get_style(), cr,
			Gtk.StateType.NORMAL, Gtk.ShadowType.IN,
			window, "tooltip",
			rect.x, rect.y, rect.width, rect.height)
	def expose_event(self, window, event):
		rect = window.get_allocation()
		window.style.paint_box(window.window,
			Gtk.StateType.NORMAL, Gtk.ShadowType.IN,
			None, window, "tooltip",
			rect.x, rect.y, rect.width, rect.height)

		return False




# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from gi.repository import Gtk
from gi.repository import GObject
from gi.repository import Pango
from blueman.gui.DeviceSelectorWidget import DeviceSelectorWidget
from blueman.Functions import setup_icon_path, spawn
from blueman.main.Config import Config


class NstBluetooth:
	def __init__(self):
		setup_icon_path()
		
		self.list = DeviceSelectorWidget()
		self.list.show()
		self.list.List.connect("row-activated", self.on_row_activated)
		self.list.List.connect("device-selected", self.on_device_selected)

		

		self.config = Config("transfer")

		self.device = None

		self.list.set_size_request(240, 280)

		self.button = Gtk.ToggleButton()

		box = Gtk.HBox()
		self.button.add(box)

		self.button_image = Gtk.Image()
		self.button_label = Gtk.Label()
		self.button_label.props.use_markup = True
		self.button_label.props.ellipsize = Pango.EllipsizeMode.END

		if self.config.props.last_device == None:
			self.list.List.set_cursor((0,))
		else:
			iter = self.list.List.find_device(self.config.props.last_device)
			if iter:
				self.list.List.set_cursor(self.list.List.get_model().get_path(iter))
			
		
		box.pack_start(self.button_image, False, True)
		box.pack_start(Gtk.VSeparator, False, True, 4)
		box.pack_start(self.button_label, True, True, 0)

		self.button.show_all()

		
		self.button.connect("toggled", self.on_button_toggled)
		
		self.wd = Gtk.Window(Gtk.WindowType.POPUP)

		self.wd.props.decorated = False
		self.wd.props.skip_pager_hint = True
		self.wd.props.skip_taskbar_hint = True
		#self.wd.props.modal = True
		self.wd.connect("button_press_event", self.on_button_press)

		self.wd.add(self.list)
		self.wd.realize()

	def on_row_activated(self, treeview, path, column):
		self.button.props.active = False

	def on_device_selected(self, treeview, device, iter):
		self.config.props.last_device = str(device.Address)
		
		self.button_label.props.label = "<b>%s</b> (%s)" % (device.Alias, device.Address)
		self.button_label.props.tooltip_markup = self.button_label.props.label
		self.button_image.props.icon_name = device.Icon

		self.device = device
	

	def on_button_press(self, widget, data=None):
		
		if data == None or data.window == None:
			return False
		
		child = data.window.get_user_data()
		if child != widget:
			while child:
				if child == widget:
				    return False
				child = child.parent
				
		self.button.props.active = False
		return True

	def popup_grab_on_window(self, window, activate_time):
		if Gdk.pointer_grab(window, True, Gdk.EventMask.BUTTON_PRESS_MASK 
								| Gdk.EventMask.BUTTON_RELEASE_MASK
								| Gdk.EventMask.POINTER_MOTION_MASK, 
								None, None, activate_time) == 0:
			if Gdk.keyboard_grab (window, True, activate_time) == 0:
				return True
			else:
				Gdk.pointer_ungrab(activate_time)
				return False
		return False


	def on_button_toggled(self, button):
		
		if button.props.active:
			if not self.popup_grab_on_window(button.window, Gtk.get_current_event_time()):
				print 'error during grab'
				return


			x, y = button.window.get_origin()
			x += button.allocation[0]
			y += button.allocation[1] + button.allocation[3]

			self.wd.move(x, y)
			self.list.set_size_request(button.allocation[2], 280)

			self.wd.grab_add()
			self.wd.show()
			self.wd.grab_focus()
			
			self.popup_grab_on_window(self.wd.window, Gtk.get_current_event_time())
			
		else:
			self.wd.hide()
			self.wd.grab_remove()


	def get_contacts_widget(self):
		return self.button

	def send_files(self, files):
		spawn(["blueman-sendto", "-d", self.device.Address] + files)

		return True

	def destroy(self):
		pass



from gi.repository import GObject
from gi.repository import Gtk
from blueman.Sdp import *
from blueman.Functions import *
from blueman.main.SignalTracker import SignalTracker
from blueman.gui.manager.ManagerProgressbar import ManagerProgressbar
from blueman.main.Config import Config
from blueman.main.AppletService import AppletService
from blueman.gui.MessageArea import MessageArea
from blueman.bluez.BlueZInterface import BlueZInterface

from blueman.Lib import rfcomm_list


def get_x_icon(icon_name, size):
    ic = get_icon(icon_name, size)
    x = get_icon("blueman-x", size)
    pixbuf = composite_icon(ic, [(x, 0, 0, 255)])

    return pixbuf


class ManagerDeviceMenu(Gtk.Menu):
    __ops__ = {}
    __instances__ = []

    def __init__(self, blueman):
        GObject.GObject.__init__(self)
        self.Blueman = blueman
        self.SelectedDevice = None

        self.is_popup = False

        #object, args,
        self.Signals = SignalTracker()
        self.MainSignals = SignalTracker()

        self.MainSignals.Handle("gobject", self.Blueman.List, "device-property-changed",
                                self.on_device_property_changed)

        ManagerDeviceMenu.__instances__.append(self)

        self.Generate()

    def __del__(self):
        dprint("deleting devicemenu")

    #	GObject.GObject.__del__(self)


    def popup(self, *args):
        self.is_popup = True

        self.MainSignals.DisconnectAll()
        self.MainSignals.Handle("gobject", self.Blueman.List, "device-property-changed",
                                self.on_device_property_changed)

        def disconnectall(x):
            self.MainSignals.DisconnectAll()

        self.MainSignals.Handle("gobject", self, "selection-done", disconnectall)

        self.Generate()

        Gtk.Menu.popup(self, *args)

    def clear(self):
        self.Signals.DisconnectAll()

        def each(child, data):
            self.remove(child)
            child.destroy()

        self.foreach(each, None)

    def set_op(self, device, message):
        ManagerDeviceMenu.__ops__[device.get_object_path()] = message
        for inst in ManagerDeviceMenu.__instances__:
            dprint("op: regenerating instance", inst)
            if inst.SelectedDevice == self.SelectedDevice and not (inst.is_popup and not inst.props.visible):
                inst.Generate()


    def get_op(self, device):
        try:
            return ManagerDeviceMenu.__ops__[device.get_object_path()]
        except:
            return None

    def unset_op(self, device):
        del ManagerDeviceMenu.__ops__[device.get_object_path()]
        for inst in ManagerDeviceMenu.__instances__:
            dprint("op: regenerating instance", inst)
            if inst.SelectedDevice == self.SelectedDevice and not (inst.is_popup and not inst.props.visible):
                inst.Generate()


    def service_property_changed(self, key, value):
        if key == "Connected":
            self.Generate()


    def on_connect(self, item, device, service_id=None, *args):
        def success(*args2):
            try:
                uuid16 = sdp_get_serial_type(device.Address, args[0])
            except:
                uuid16 = 0

            dprint("success", args2)
            prog.message(_("Success!"))

            if service_id == "serial" and SERIAL_PORT_SVCLASS_ID in uuid16:
                MessageArea.show_message(_("Serial port connected to %s") % args2[0], Gtk.STOCK_DIALOG_INFO)
            else:
                MessageArea.close()

            self.unset_op(device)

        def fail(*args):
            prog.message(_("Failed"))

            self.unset_op(device)
            dprint("fail", args)
            MessageArea.show_message(_("Connection Failed: ") + e_(str(args[0])))

        self.set_op(device, _("Connecting..."))
        prog = ManagerProgressbar(self.Blueman, False)

        try:
            appl = AppletService()
        except:
            dprint("** Failed to connect to applet")
            fail()
            return
        try:
            appl.SetTimeHint(Gtk.get_current_event_time())
        except:
            pass

        if service_id:
            svc = device.Services[service_id]

            if service_id == "network":
                uuid = args[0]
                appl.ServiceProxy(svc.get_interface_name(),
                                  svc.get_object_path(),
                                  "Connect",
                                  [uuid],
                                  reply_handler=success,
                                  error_handler=fail, timeout=200)

            elif service_id == "input":
                appl.ServiceProxy(svc.get_interface_name(),
                                  svc.get_object_path(),
                                  "Connect", [],
                                  reply_handler=success,
                                  error_handler=fail, timeout=200)

            elif service_id == "serial":
                uuid = str(args[0])

                appl.RfcommConnect(device.get_object_path(),
                                   uuid,
                                   reply_handler=success,
                                   error_handler=fail, timeout=200)

            else:
                appl.ServiceProxy(svc.get_interface_name(),
                                  svc.get_object_path(),
                                  "Connect", [],
                                  reply_handler=success,
                                  error_handler=fail, timeout=200)
        else:
            appl.ServiceProxy(device.get_interface_name(), device.get_object_path(), 'Connect', [],
                              reply_handler=success, error_handler=fail, timeout=200)

        prog.start()

    def on_disconnect(self, item, device, service_id=None, *args):
        if service_id == "serial":
            try:
                appl = AppletService()
            except:
                dprint("** Failed to connect to applet")
            else:
                appl.RfcommDisconnect(device.get_object_path(), args[0])
                self.Generate()
        else:
            try:
                appl = AppletService()
            except:
                dprint("** Failed to connect to applet")
                return
            if service_id:
                connection_object = device.Services[service_id]
            else:
                connection_object = device
            appl.ServiceProxy(connection_object.get_interface_name(),
                              connection_object.get_object_path(), "Disconnect", [])


    def on_device_property_changed(self, List, device, iter, (key, value)):
    #		print "menu:", key, value
        if List.compare(iter, List.selected()):
            if key == "Connected" \
                or key == "Fake" \
                or key == "UUIDs" \
                or key == "Trusted" \
                or key == "Paired":
                self.Generate()

    def Generate(self):
        self.clear()

        appl = AppletService()

        items = []

        if not self.is_popup or self.props.visible:
            selected = self.Blueman.List.selected()
            if not selected:
                return
            device = self.Blueman.List.get(selected, "device")["device"]
        else:
            (x, y) = self.Blueman.List.get_pointer()
            path = self.Blueman.List.get_path_at_pos(x, y)
            if path != None:
                device = self.Blueman.List.get(path[0], "device")["device"]
            else:
                return

        if not device.Valid:
            return
        self.SelectedDevice = device

        op = self.get_op(device)

        if op != None:
            item = create_menuitem(op, get_icon("gtk-connect", 16))
            item.props.sensitive = False
            item.show()
            self.append(item)
            return

        rets = self.Blueman.Plugins.Run("on_request_menu_items", self, device)

        for ret in rets:
            if ret:
                for (item, pos) in ret:
                    items.append((pos, item))

        if device.Fake:
            item = create_menuitem(_("_Add Device"), get_icon("gtk-add", 16))
            self.Signals.Handle("gobject", item, "activate",
                                lambda x: self.Blueman.add_device(device))
            item.show()
            self.append(item)
            item.props.tooltip_text = _("Add this device to known devices list")

            item = create_menuitem(_("_Setup..."), get_icon("gtk-properties", 16))
            self.append(item)
            self.Signals.Handle("gobject", item, "activate",
                                lambda x: self.Blueman.setup(device))
            item.show()
            item.props.tooltip_text = _("Run the setup assistant for this device")

            item = create_menuitem(_("_Pair"), get_icon("gtk-dialog-authentication", 16))
            self.Signals.Handle("gobject", item, "activate",
                                lambda x: self.Blueman.bond(device))
            self.append(item)
            item.show()
            item.props.tooltip_text = _("Pair with the device")

            item = Gtk.SeparatorMenuItem()
            item.show()
            self.append(item)

            send_item = create_menuitem(_("Send a _File..."), get_icon("gtk-copy", 16))
            self.Signals.Handle("gobject", send_item, "activate",
                                lambda x: self.Blueman.send(device))
            send_item.show()
            self.append(send_item)



        else:
            dprint(device.Alias)

            item = None

            have_disconnectables = False
            have_connectables = False

            if True in map(lambda x: x[0] >= 100 and x[0] < 200, items):
                have_disconnectables = True

            if True in map(lambda x: x[0] < 100, items):
                have_connectables = True

            if True in map(lambda x: x[0] >= 200, items) and (have_connectables or have_disconnectables):
                item = Gtk.SeparatorMenuItem()
                item.show()
                items.append((199, item))

            if have_connectables:
                item = Gtk.MenuItem()
                label = Gtk.Label()
                label.set_markup(_("<b>Connect To:</b>"))
                label.props.xalign = 0.0

                label.show()
                item.add(label)
                item.props.sensitive = False
                item.show()
                items.append((0, item))

            if have_disconnectables:
                item = Gtk.MenuItem()
                label = Gtk.Label()
                label.set_markup(_("<b>Disconnect:</b>"))
                label.props.xalign = 0.0

                label.show()
                item.add(label)
                item.props.sensitive = False
                item.show()
                items.append((99, item))

            items.sort(lambda a, b: cmp(a[0], b[0]))
            for priority, item in items:
                self.append(item)

            if items != []:
                item = Gtk.SeparatorMenuItem()
                item.show()
                self.append(item)

            del items

            send_item = create_menuitem(_("Send a _File..."), get_icon("gtk-copy", 16))
            send_item.props.sensitive = False
            self.append(send_item)
            send_item.show()

            browse_item = create_menuitem(_("_Browse Device..."), get_icon("gtk-open", 16))
            browse_item.props.sensitive = False
            self.append(browse_item)
            browse_item.show()

            uuids = device.UUIDs
            for uuid in uuids:
                uuid16 = uuid128_to_uuid16(uuid)
                if uuid16 == OBEX_OBJPUSH_SVCLASS_ID:
                    self.Signals.Handle("gobject", send_item, "activate", lambda x: self.Blueman.send(device))
                    send_item.props.sensitive = True

                if uuid16 == OBEX_FILETRANS_SVCLASS_ID:
                    self.Signals.Handle("gobject", browse_item, "activate", lambda x: self.Blueman.browse(device))
                    browse_item.props.sensitive = True

            item = Gtk.SeparatorMenuItem()
            item.show()
            self.append(item)

            item = create_menuitem(_("_Pair"), get_icon("gtk-dialog-authentication", 16))
            item.props.tooltip_text = _("Create pairing with the device")
            self.append(item)
            item.show()
            if not device.Paired:
                self.Signals.Handle("gobject", item, "activate", lambda x: self.Blueman.bond(device))
            else:
                item.props.sensitive = False

            if not device.Trusted:
                item = create_menuitem(_("_Trust"), get_icon("blueman-trust", 16))
                self.Signals.Handle("gobject", item, "activate", lambda x: self.Blueman.toggle_trust(device))
                self.append(item)
                item.show()
            else:
                item = create_menuitem(_("_Untrust"), get_icon("blueman-untrust", 16))
                self.append(item)
                self.Signals.Handle("gobject", item, "activate", lambda x: self.Blueman.toggle_trust(device))
                item.show()
            item.props.tooltip_text = _("Mark/Unmark this device as trusted")

            item = create_menuitem(_("_Setup..."), get_icon("gtk-properties", 16))
            self.append(item)
            self.Signals.Handle("gobject", item, "activate", lambda x: self.Blueman.setup(device))
            item.show()
            item.props.tooltip_text = _("Run the setup assistant for this device")

            if BlueZInterface.get_interface_version()[0] < 5:
                def update_services(item):
                    def reply():
                        self.unset_op(device)
                        prog.message(_("Success!"))
                        MessageArea.close()

                    def error(*args):
                        self.unset_op(device)
                        prog.message(_("Fail"))
                        MessageArea.show_message(e_(str(args[0])))

                    prog = ManagerProgressbar(self.Blueman, False, _("Refreshing"))
                    prog.start()
                    self.set_op(device, _("Refreshing Services..."))
                    appl.RefreshServices(device.get_object_path(), reply_handler=reply, error_handler=error)

                item = create_menuitem(_("Refresh Services"), get_icon("gtk-refresh", 16))
                self.append(item)
                self.Signals.Handle(item, "activate", update_services)
                item.show()

            item = Gtk.SeparatorMenuItem()
            item.show()
            self.append(item)

            item = create_menuitem(_("_Remove..."), get_icon("gtk-delete", 16))
            self.Signals.Handle(item, "activate", lambda x: self.Blueman.remove(device))
            self.append(item)
            item.show()
            item.props.tooltip_text = _("Remove this device from the known devices list")

            item = Gtk.SeparatorMenuItem()
            item.show()
            self.append(item)

            item = create_menuitem(_("_Disconnect"), get_icon("gtk-disconnect", 16))
            item.props.tooltip_text = _("Forcefully disconnect the device")

            self.append(item)
            item.show()

            def on_disconnect(item):
                def finished(*args):
                    self.unset_op(device)

                self.set_op(device, _("Disconnecting..."))
                self.Blueman.disconnect(device,
                                        reply_handler=finished,
                                        error_handler=finished)

            if device.Connected:
                self.Signals.Handle(item, "activate", on_disconnect)

            else:
                item.props.sensitive = False



from blueman.Functions import *
from blueman.Constants import HAL_ENABLED
from blueman.plugins.AppletPlugin import AppletPlugin
from blueman.main.Mechanism import Mechanism
from blueman.main.Config import Config
from blueman.gui.Notification import Notification
from blueman.Sdp import *
from blueman.bluez.Network import Network

from blueman.main.SignalTracker import SignalTracker

import blueman.bluez as Bluez

from gi.repository import GObject
from gi.repository import Gtk

if not HAL_ENABLED:
    raise ImportError("NMIntegration (deprecated) requires hal support")


class NMIntegration(AppletPlugin):
    __description__ = _("<b>Deprecated</b>\nMakes DUN/PAN connections available for NetworkManager 0.7")
    __icon__ = "modem"
    __depends__ = ["DBusService"]
    __conflicts__ = ["PPPSupport", "DhcpClient"]
    __author__ = "Walmis"
    if HAL_ENABLED:
        __priority__ = 2

    def on_load(self, applet):
        self.Signals = SignalTracker()

        self.Signals.Handle('bluez', Network(), self.on_network_prop_changed, 'PropertyChanged', path_keyword='path')

    def on_unload(self):
        self.Signals.DisconnectAll()

    def on_network_prop_changed(self, key, value, path):
        if key == "Interface":
            if value != "":
                m = Mechanism()
                m.HalRegisterNetDev(value)

    #in: bluez_device_path, rfcomm_device
    #@dbus.service.method(dbus_interface='org.blueman.Applet', in_signature="ss", out_signature="")
    def RegisterModem(self, device_path, rfcomm_device):
        dev = Bluez.Device(device_path)
        props = dev.get_properties()

        m = Mechanism()

        def reply():
            dprint("Registered modem")

        def err(excp):
            d = Gtk.MessageDialog(None, type=Gtk.MessageType.WARNING)
            d.props.icon_name = "blueman"
            d.props.text = _("CDMA or GSM not supported")
            d.props.secondary_text = _(
                "The device %s does not appear to support GSM/CDMA.\nThis connection will not work.") % props["Alias"]

            d.add_button(Gtk.STOCK_OK, Gtk.ResponseType.NO)
            resp = d.run()
            d.destroy()

        m.HalRegisterModemPort(rfcomm_device, props["Address"], reply_handler=reply, error_handler=err)


    #in: bluez_device_path, rfcomm_device
    #@dbus.service.method(dbus_interface='org.blueman.Applet', in_signature="s", out_signature="")
    def UnregisterModem(self, device):
        m = Mechanism()
        m.HalUnregisterModemPortDev(device)

        dprint("Unregistered modem")

    def on_rfcomm_connected(self, device, port, uuid):
        signals = SignalTracker()

        def modem_added(mon, udi, address):
            if device.Address == address:
                dprint(udi)
                device.udi = udi

        def modem_removed(mon, udi):
            if device.udi == udi:
                dprint(udi)
                signals.DisconnectAll()

        def disconnected(mon, udi):
            device.Services["serial"].Disconnect(port)
            self.UnregisterModem(port)

        def device_propery_changed(key, value):
            if key == "Connected" and not value:
                self.UnregisterModem(port)


        uuid16 = sdp_get_serial_type(device.Address, uuid)
        if DIALUP_NET_SVCLASS_ID in uuid16:
            try:
                signals.Handle(self.Applet.Plugins.NMMonitor, "modem-added", modem_added)
                signals.Handle(self.Applet.Plugins.NMMonitor, "modem-removed", modem_removed)
                signals.Handle(self.Applet.Plugins.NMMonitor, "disconnected", disconnected)
            except KeyError:
                pass

            signals.Handle("bluez", device.Device, device_propery_changed, "PropertyChanged")
            self.RegisterModem(device.get_object_path(), port)

    def rfcomm_connect_handler(self, device, uuid, reply_handler, error_handler):
        uuid16 = sdp_get_serial_type(device.Address, uuid)
        if DIALUP_NET_SVCLASS_ID in uuid16:
            device.Services["serial"].Connect(uuid, reply_handler=reply_handler, error_handler=error_handler)
            return True
        else:
            return False


    def on_rfcomm_disconnect(self, port):
        self.UnregisterModem(port)



# Copyright (C) 2009 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from blueman.plugins.AppletPlugin import AppletPlugin
import dbus
from gi.repository import GObject
from blueman.main.SignalTracker import SignalTracker
from blueman.gui.Notification import Notification
from blueman.Sdp import *
from blueman.Functions import get_icon, composite_icon
import weakref

class ConnectionHandler:
	def __init__(self, parent, device, uuid, reply, err):
		self.parent = parent
		self.device = device
		self.uuid = uuid
		self.reply = reply
		self.err = err
		self.rfcomm_dev = None
		self.timeout = None
		
		self.signals = SignalTracker()
		self.signals.Handle("dbus", self.parent.bus, 
				    self.on_mm_device_added, 
				    "DeviceAdded", 
				    "org.freedesktop.ModemManager")
				    
		#for some reason these handlers take a reference and don't give it back
		#so i have to workaround :(
		w = weakref.ref(self)
		device.Services["serial"].Connect(uuid, 
						  reply_handler=lambda *args: w() and w().on_connect_reply(*args), 
						  error_handler=lambda *args: w() and w().on_connect_error(*args))
		
	def __del__(self):
		dprint("deleting")
		
	def on_connect_reply(self, rfcomm):
		self.rfcomm_dev = rfcomm
		self.timeout = GObject.timeout_add(10000, self.on_timeout)
		
	def on_connect_error(self, *args):
		self.err(*args)
		self.cleanup()
		
	def cleanup(self):
		if self.timeout:
			GObject.source_remove(self.timeout)
		self.signals.DisconnectAll()
		
		del self.device
		
	def on_mm_device_added(self, path):
		dprint(path)
		props = self.parent.bus.call_blocking("org.freedesktop.ModemManager", 
						path, 
						"org.freedesktop.DBus.Properties", 
						"GetAll", 
						"s", 
						["org.freedesktop.ModemManager.Modem"])
						
		if self.rfcomm_dev and props["Driver"] == "bluetooth" and props["Device"] in self.rfcomm_dev:
			dprint("It's our bluetooth modem!")
			
			modem = get_icon("modem", 24)
			blueman = get_icon("blueman", 48)
			
			icon = composite_icon(blueman, [(modem, 24, 24, 255)])
			
			Notification(_("Bluetooth Dialup"), 
				     _("DUN connection on %s will now be available in Network Manager") % self.device.Alias, 
				     pixbuf=icon,
				     status_icon=self.parent.Applet.Plugins.StatusIcon)
				     
			self.reply(self.rfcomm_dev)
			self.cleanup()

			
	def on_timeout(self):
		self.timeout = None
		self.err(dbus.DBusException(_("Modem Manager did not support the connection")))
		self.cleanup()

class NMDUNSupport(AppletPlugin):
	__depends__ = ["StatusIcon", "DBusService"]
	__conflicts__ = ["PPPSupport", "NMIntegration"]
	__icon__ = "modem"
	__author__ = "Walmis"
	__description__ = _("Provides support for Dial Up Networking (DUN) with ModemManager and NetworkManager 0.8")
	__priority__ = 1
	
	def on_load(self, applet):
		self.bus = dbus.SystemBus()
		
	def on_unload(self):
		pass
		
	def rfcomm_connect_handler(self, device, uuid, reply, err):
		uuid16 = sdp_get_serial_type(device.Address, uuid)
		if DIALUP_NET_SVCLASS_ID in uuid16:		

			ConnectionHandler(self, device, uuid, reply, err)
		
			return True
		else:
			return False




from PropertiesBlueZInterface import PropertiesBlueZInterface
from errors import raise_dbus_error


class ServiceInterface(PropertiesBlueZInterface):
    @raise_dbus_error
    def __init__(self, interface, obj_path, methods):
        self.methods = methods
        super(ServiceInterface, self).__init__(interface, obj_path)

    def __getattribute__(self, name):
        if name in object.__getattribute__(self, 'methods'):
            func = getattr(self.get_interface(), name)
            return raise_dbus_error(func)
        else:
            return super(ServiceInterface, self).__getattribute__(name)


# Copyright (C) 2009 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2009 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from gi.repository import GObject
import os
from blueman.Functions import dprint

import blueman.plugins.config
from blueman.plugins.ConfigPlugin import ConfigPlugin

print "Loading configuration plugins"

path = os.path.dirname(blueman.plugins.config.__file__)
plugins = []
for root, dirs, files in os.walk(path):
	for f in files:
		if f.endswith(".py") and not (f.endswith(".pyc") or f.endswith("_.py")):
			plugins.append(f[0:-3])

for plugin in plugins:
	try:
		__import__("blueman.plugins.config.%s" % plugin, None, None, [])
	except ImportError, e:
		dprint("Skipping plugin %s\n%s" % (plugin, e))

def compare(a, b):
	return cmp(a.__priority__, b.__priority__)

class Config(object):
	def __new__(c, section=""):
		classes = ConfigPlugin.__subclasses__()
		classes.sort(compare)
		
		for cls in classes:
			try:
				inst = cls(section)
				print "Using %s config backend" % cls.__plugin__
				return inst
			except Exception, e:
				print "Skipping plugin", cls.__plugin__
				print e
			
			print "No suitable configuration backend found, exitting"
			exit(1)



from blueman.main.SpeedCalc import SpeedCalc
from blueman.main.Config import Config
from blueman.ods.OdsManager import OdsManager
from blueman.main.Device import Device
from blueman.Functions import *
from blueman.Lib import get_special_dir, SpecialDirType
import os
from gi.repository import GObject
from blueman.gui.Notification import Notification
import weakref


class Transfer(OdsManager):
    def __init__(self, applet):
        OdsManager.__init__(self)
        self.Applet = applet
        try:
            self.status_icon = self.Applet.Plugins.StatusIcon
        except:
            self.status_icon = None

        self.GHandle("server-created", self.on_server_created)
        self.Config = Config("transfer")

        #check options
        if self.Config.props.opp_enabled == None:
            self.Config.props.opp_enabled = True

        if self.Config.props.ftp_enabled == None:
            self.Config.props.ftp_enabled = True

        self.create_server("opp")
        self.create_server("ftp")

        self.allowed_devices = []

    def create_server(self, pattern):

        if pattern == "opp":
            if self.Config.props.opp_enabled:
                OdsManager.create_server(self)
        elif pattern == "ftp":
            if self.Config.props.ftp_enabled:
                OdsManager.create_server(self, pattern="ftp", require_pairing=True)


    def start_server(self, pattern):
        server = self.get_server(pattern)
        if server != None:
            if self.Config.props.shared_path == None:
                d = get_special_dir(SpecialDirType.PUBLIC_SHARE)
                if d == None:
                    self.Config.props.shared_path = os.path.expanduser("~")
                else:
                    self.Config.props.shared_path = d

            if not os.path.isdir(self.Config.props.shared_path):
                raise Exception("Configured share directory %s does not exist" % self.Config.props.shared_path)

            if pattern == "opp":
                server.Start(self.Config.props.shared_path, True, False)
            elif pattern == "ftp":
                if self.Config.props.ftp_allow_write == None:
                    self.Config.props.ftp_allow_write = False

                server.Start(self.Config.props.shared_path, self.Config.props.ftp_allow_write, True)
            return True
        else:
            return False

    def on_server_created(self, inst, server, pattern):
        def on_started(server):
            dprint(pattern, "Started")

        server.GHandle("started", on_started)
        server.GHandle("session-created", self.on_session_created)
        server.pattern = pattern
        self.start_server(pattern)


    def on_session_created(self, server, session):
        dprint(server.pattern, "session created")
        if server.pattern != "opp":
            return

        session.GHandle("transfer-progress", self.transfer_progress)
        session.GHandle("cancelled", self.transfer_finished, "cancelled")
        session.GHandle("disconnected", self.transfer_finished, "disconnected")
        session.GHandle("transfer-completed", self.transfer_finished, "completed")
        session.GHandle("error-occurred", self.transfer_finished, "error")
        session.GHandle("transfer-started", self.on_transfer_started)

        session.transfer = {}
        session.transfer["notification"] = None
        session.transfer["silent_transfers"] = 0
        session.transfer["normal_transfers"] = 0

        session.server = server


    def on_transfer_started(self, session, filename, local_path, total_bytes):
        dprint("transfer started", filename)
        info = session.server.GetServerSessionInfo(session.object_path)
        trusted = False
        try:
            dev = self.Applet.Manager.get_adapter().find_device(info["BluetoothAddress"])
            dev = Device(dev)
            name = dev.Alias
            trusted = dev.Trusted
        except Exception as e:
            dprint(e)
            name = info["BluetoothAddress"]

        wsession = weakref.proxy(session)
        wself = weakref.proxy(self)

        icon = get_icon("blueman", 48)

        session.transfer["filename"] = filename
        session.transfer["filepath"] = local_path
        session.transfer["total"] = total_bytes
        session.transfer["finished"] = False
        session.transfer["failed"] = False
        session.transfer["waiting"] = True

        session.transfer["address"] = info["BluetoothAddress"]
        session.transfer["name"] = name

        session.transfer["transferred"] = 0

        def access_cb(n, action):
            dprint(action)

            if action == "closed":
                if wsession.transfer["waiting"]:
                    wsession.Reject()

            if wsession.transfer["waiting"]:
                if action == "accept":
                    wsession.Accept()
                    wself.allowed_devices.append(wsession.transfer["address"])
                    GObject.timeout_add(60000, wself.allowed_devices.remove, wsession.transfer["address"])
                else:
                    wsession.Reject()
                wsession.transfer["waiting"] = False

        if info["BluetoothAddress"] not in self.allowed_devices and not (self.Config.props.opp_accept and trusted):

            n = Notification(_("Incoming file over Bluetooth"),
                             _("Incoming file %(0)s from %(1)s") % {"0": "<b>" + os.path.basename(filename) + "</b>",
                                                                    "1": "<b>" + name + "</b>"},
                             30000, [["accept", _("Accept"), "gtk-yes"], ["reject", _("Reject"), "gtk-no"]], access_cb,
                             icon, self.status_icon)

            if total_bytes > 350000:
                session.transfer["normal_transfers"] += 1
            else:
                session.transfer["silent_transfers"] += 1
        else:
            if total_bytes > 350000:
                n = Notification(_("Receiving file"),
                                 _("Receiving file %(0)s from %(1)s") % {
                                 "0": "<b>" + os.path.basename(filename) + "</b>", "1": "<b>" + name + "</b>"},
                                 pixbuf=icon, status_icon=self.status_icon)

                session.transfer["normal_transfers"] += 1
            else:
                session.transfer["silent_transfers"] += 1
                n = None

            access_cb(n, "accept")

        session.transfer["notification"] = n

    def transfer_progress(self, session, bytes_transferred):
        session.transfer["transferred"] = bytes_transferred

    def add_open(self, n, name, path):
        if Notification.actions_supported():
            print("adding action")

            def on_open(*args):
                print("open")
                spawn(["xdg-open", path], True)

            n.add_action("open", name, on_open, None)
            n.show()


    def transfer_finished(self, session, *args):
        type = args[-1]
        dprint(args)
        try:
            if not session.transfer["finished"]:

                if type != "cancelled" and type != "error":
                    session.transfer["finished"] = True

                    if session.transfer["total"] > 350000:
                        icon = get_icon("blueman", 48)
                        n = Notification(_("File received"),
                                         _("File %(0)s from %(1)s successfully received") % {
                                         "0": "<b>" + session.transfer["filename"] + "</b>",
                                         "1": "<b>" + session.transfer["name"] + "</b>"},
                                         pixbuf=icon, status_icon=self.status_icon)
                        self.add_open(n, "Open", session.transfer["filepath"])
                else:
                    session.transfer["failed"] = True
                    session.transfer["finished"] = True

                    n = session.transfer["notification"]
                    if n:
                        n.close()

                    icon = get_icon("blueman", 48)

                    session.transfer["notification"] = Notification(_("Transfer failed"),
                                                                    _("Transfer of file %(0)s failed") % {
                                                                    "0": "<b>" + session.transfer["filename"] + "</b>",
                                                                    "1": "<b>" + session.transfer["name"] + "</b>"},
                                                                    pixbuf=icon, status_icon=self.status_icon)
                    if session.transfer["total"] > 350000:
                        session.transfer["normal_transfers"] -= 1
                    else:
                        session.transfer["silent_transfers"] -= 1

            if type == "disconnected":
                icon = get_icon("blueman", 48)

                if session.transfer["normal_transfers"] == 0 and session.transfer["silent_transfers"] == 1:
                    n = Notification(_("File received"),
                                     _("File %(0)s from %(1)s successfully received") % {
                                     "0": "<b>" + session.transfer["filename"] + "</b>",
                                     "1": "<b>" + session.transfer["name"] + "</b>"},
                                     pixbuf=icon, status_icon=self.status_icon)

                    self.add_open(n, "Open", session.transfer["filepath"])

                elif session.transfer["normal_transfers"] == 0 and session.transfer["silent_transfers"] > 0:
                    n = Notification(_("Files received"),
                                     ngettext("Received %d file in the background",
                                              "Received %d files in the background",
                                              session.transfer["silent_transfers"]) % session.transfer[
                                         "silent_transfers"],
                                     pixbuf=icon, status_icon=self.status_icon)

                    self.add_open(n, "Open Location", self.Config.props.shared_path)

                elif session.transfer["normal_transfers"] > 0 and session.transfer["silent_transfers"] > 0:

                    n = Notification(_("Files received"),
                                     ngettext("Received %d more file in the background",
                                              "Received %d more files in the background",
                                              session.transfer["silent_transfers"]) % session.transfer[
                                         "silent_transfers"],
                                     pixbuf=icon, status_icon=self.status_icon)
                    self.add_open(n, "Open Location", self.Config.props.shared_path)

                del session.transfer
                del session.server

        except KeyError:
            pass

    def on_server_destroyed(self, inst, server):
        pass



# Copyright (C) 2010 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import traceback

class MethodAlreadyExists(Exception):
	pass

class BasePlugin(object):
	__depends__ = []
	__conflicts__ = []
	__priority__ = 0
	
	__description__ = None
	__author__ = None
	
	__unloadable__ = True
	__autoload__ = True
	
	__instance__ = None
	
	def __init__(self, parent):
		self.__parent__ = parent
		
		self.__methods = []
		
	def __del__(self):
		print "Deleting plugin instance", self
		
	
	@classmethod
	def add_method(cls, func):
		func.im_self.__methods.append((cls, func.__name__))

		if func.__name__ in cls.__dict__:
			raise MethodAlreadyExists
		else:
			setattr(cls, func.__name__, func)	
	
	def _unload(self):
		self.on_unload()

		for cls, met in self.__methods:
			delattr(cls, met)
			
		self.__class__.__instance__ = None
	
	def _load(self, parent):
		try:
			self.on_load(parent)
			#self.on_manager_state_changed(applet.Manager != None)
			self.__class__.__instance__ = self
		except Exception, e:
			#AppletPlugin.instances.remove(self)
			self.__class__.__instance__ = None
			traceback.print_exc()
			raise
			
	#virtual methods
	def on_load(self, applet):
		pass
	
	def on_unload(self):
		raise NotImplementedError



from blueman.Functions import dprint

from gi.repository import Gtk
from gi.repository import GObject
import os
from blueman.bluez.Adapter import Adapter
from blueman.Constants import *
from blueman.gui.DeviceSelectorList import DeviceSelectorList


class DeviceSelectorWidget(Gtk.VBox):
    def __init__(self, adapter=None):

        GObject.GObject.__init__(self)

        self.props.spacing = 1
        self.set_size_request(360, 340)

        sw = Gtk.ScrolledWindow()
        self.List = devlist = DeviceSelectorList(adapter)
        if self.List.Adapter != None:
            self.List.DisplayKnownDevices()

        sw.add(devlist)
        sw.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC)
        sw.set_shadow_type(Gtk.ShadowType.IN)

        self.Builder = Gtk.Builder()
        self.Builder.add_from_file(UI_PATH + "/device-list-widget.ui")

        sitem = self.Builder.get_object("search")

        self.cb_adapters = self.Builder.get_object("adapters")

        cell = Gtk.CellRendererText()

        self.cb_adapters.pack_start(cell, True)
        self.cb_adapters.connect("changed", self.on_adapter_selected)
        self.cb_adapters.add_attribute(cell, 'text', 0)

        button = self.Builder.get_object("b_search")
        button.connect("clicked", self.on_search_clicked)

        self.pbar = self.Builder.get_object("progressbar1")

        self.List.connect("discovery-progress", self.on_discovery_progress)

        self.pack_start(sw, True, False, 0)
        self.pack_start(sitem, False, False, 0)

        sitem.show()

        sw.show_all()

        self.List.connect("adapter-changed", self.on_adapter_changed)
        self.List.connect("adapter-added", self.on_adapter_added)
        self.List.connect("adapter-removed", self.on_adapter_removed)
        self.List.connect("adapter-property-changed", self.on_adapter_prop_changed)

        self.update_adapters_list()

    def __del__(self):
        self.List.destroy()
        dprint("Deleting widget")

    def on_discovery_progress(self, devlist, fraction):
        self.pbar.props.fraction = fraction

    def on_search_clicked(self, button):
        self.List.DiscoverDevices()

    def on_adapter_prop_changed(self, devlist, adapter, (key, value)):
        if key == "Name":
            self.update_adapters_list()
        elif key == "Discovering":
            if not value:
                self.pbar.props.fraction = 0

    def on_adapter_added(self, devlist, adapter_path):
        self.update_adapters_list()

    def on_adapter_removed(self, devlist, adapter_path):
        self.update_adapters_list()

    def on_adapter_selected(self, cb_adapters):
        dprint("selected")
        iter = cb_adapters.get_active_iter()
        if iter:
            adapter_path = cb_adapters.get_model().get_value(iter, 1)
            if self.List.Adapter:
                if self.List.Adapter.get_object_path() != adapter_path:
                    self.List.SetAdapter(os.path.basename(adapter_path))

    def on_adapter_changed(self, devlist, adapter_path):
        dprint("changed")
        if adapter_path is None:
            self.update_adapters_list()
        else:
            if self.List.Adapter:
                self.List.DisplayKnownDevices()

    def update_adapters_list(self):

        self.cb_adapters.get_model().clear()
        adapters = self.List.manager.list_adapters()
        num = len(adapters)
        if num == 0:
            self.cb_adapters.props.visible = False
            self.List.props.sensitive = False
        elif num == 1:
            self.cb_adapters.props.visible = False
            self.List.props.sensitive = True
        elif num > 1:
            self.List.props.sensitive = True
            self.cb_adapters.props.visible = True
            for adapter in adapters:
                props = adapter.get_properties()
                name = props["Name"]

                iter = self.cb_adapters.get_model().append([name, adapter.get_object_path()])
                if adapter.get_object_path() == self.List.Adapter.get_object_path():
                    self.cb_adapters.set_active_iter(iter)



service_cls= [
	"positioning",
	"networking",
	"rendering",
	"capturing",
	"object transfer",
	"audio",
	"telephony",
	"information"
]

major_cls = [
	"miscellaneous",
	"computer",
	"phone",
	"access point",
	"audio/video",
	"peripheral",
	"imaging",
	"wearable",
	"toy",
	"uncategorized"
]

computer_minor_cls = [
	"uncategorized",
	"desktop",
	"server",
	"laptop",
	"handheld",
	"palm",
	"wearable"
]
computer_minor_cls_i18n = [
	#translators: device class
	_("uncategorized"),
	#translators: device class
	_("desktop"),
	#translators: device class
	_("server"),
	#translators: device class
	_("laptop"),
	#translators: device class
	_("handheld"),
	#translators: device class
	_("palm"),
	"wearable"
]


phone_minor_cls = [
	"uncategorized",
	"cellular",
	"cordless",
	"smart phone",
	"modem",
	"isdn"
]
phone_minor_cls_i18n = [
	#translators: device class
	_("uncategorized"),
	#translators: device class
	_("cellular"),
	#translators: device class
	_("cordless"),
	#translators: device class
	_("smart phone"),
	#translators: device class
	_("modem"),
	#translators: device class
	_("isdn")
]
access_point_minor_cls = [
	"fully",
	"1-17 percent",
	"17-33 percent",
	"33-50 percent",
	"50-67 percent",
	"67-83 percent",
	"83-99 percent",
	"not available"
]

audio_video_minor_cls = [
	"uncategorized",
	"headset",
	"handsfree",
	"unknown",
	"microphone",
	"loudspeaker",
	"headphones",
	"portable audio",
	"car audio",
	"set-top box",
	"hifi audio",
	"vcr",
	"video camera",
	"camcorder",
	"video monitor",
	"video display and loudspeaker",
	"video conferencing",
	"unknown",
	"gaming/toy"
]
audio_video_minor_cls_i18n = [
	"uncategorized",
	#translators: device class
	_("headset"),
	#translators: device class
	_("handsfree"),
	#translators: device class
	_("unknown"),
	#translators: device class
	_("microphone"),
	"loudspeaker",
	"headphones",
	"portable audio",
	"car audio",
	"set-top box",
	"hifi audio",
	"vcr",
	"video camera",
	"camcorder",
	"video monitor",
	"video display and loudspeaker",
	"video conferencing",
	"unknown",
	"gaming/toy"
]
peripheral_minor_cls = [
	"uncategorized",
	"keyboard",
	"pointing",
	"combo"
]
peripheral_minor_cls_i18n = [
	"uncategorized",
	#translators: device class
	_("keyboard"),
	#translators: device class
	_("pointing"),
	"combo"
]

imaging_minor_cls = [
	"display",
	"camera",
	"scanner",
	"printer"
]

wearable_minor_cls = [
	"wrist watch",
	"pager",
	"jacket",
	"helmet",
	"glasses"
]

toy_minor_cls = [
	"robot",
	"vehicle",
	"doll",
	"controller",
	"game"
]

def get_major_class(klass):
	index = (klass >> 8) & 0x1F
	
	if index > 8:
		return major_cls[9]
	
	return major_cls[index]


def get_minor_class(klass, i18n=False):
	if klass == "unknown":
		if i18n:
			return _("unknown")
		else:
			return "unknown"
		
	i = (klass >> 8) & 0x1F
	
	

	if i == 1: 
		minor_index = (klass >> 2) & 0x3F;
		if minor_index < len(computer_minor_cls):
			if i18n:
				return computer_minor_cls_i18n[minor_index]
			else:
				return computer_minor_cls[minor_index]
		else:
			return ""
	elif i == 2:
		minor_index = (klass >> 2) & 0x3F;
		if (minor_index < len(phone_minor_cls)):
			if i18n:
				return phone_minor_cls_i18n[minor_index]
			else:
				return phone_minor_cls[minor_index]
		return "";
	elif i == 3:
		minor_index = (klass >> 5) & 0x07;
		if (minor_index < len(access_point_minor_cls)):
			return access_point_minor_cls[minor_index]
		else:
			return "";
	elif i == 4: 
		minor_index = (klass >> 2) & 0x3F;
		if (minor_index < len(audio_video_minor_cls)):
			if i18n:
				return audio_video_minor_cls_i18n[minor_index];
			else:
				return audio_video_minor_cls[minor_index];
		else:
			return "";
	elif i == 5:
		minor_index = (klass >> 6) & 0x03;
		if (minor_index < len(peripheral_minor_cls)):
			if i18n:
				return peripheral_minor_cls_i18n[minor_index];
			else:
				return peripheral_minor_cls[minor_index];
		else:
			return "";
	elif i == 6:
		return "imaging"
		
	elif i == 7:
		minor_index = (klass >> 2) & 0x3F;
		if (minor_index < len(wearable_minor_cls)):
			return wearable_minor_cls[minor_index];
		else:
			return "";
	elif i == 8:
		minor_index = (klass >> 2) & 0x3F;
		if (minor_index < len(toy_minor_cls)):
			return toy_minor_cls[minor_index];
		else:
			return "";

	if i18n:
		return _("unknown")
	else:
		return "unknown"




# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from gi.repository import GObject
from blueman.ods.OdsBase import OdsBase

class OdsServerSession(OdsBase):
	__gsignals__ = {
		'cancelled' : (GObject.SignalFlags.NO_HOOKS, None, ()),
		'disconnected' : (GObject.SignalFlags.NO_HOOKS, None, ()),
		'transfer-started' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT,)),
		'transfer-progress' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT,)),
		'transfer-completed' : (GObject.SignalFlags.NO_HOOKS, None, ()),
		'error-occurred' : (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT, GObject.TYPE_PYOBJECT,)),
	}
	
	def __init__(self, obj_path):
		OdsBase.__init__(self, "org.openobex.ServerSession", obj_path)
		
		self.Handle("Cancelled", self.on_cancelled)
		self.Handle("Disconnected", self.on_disconnected)
		self.Handle("TransferStarted", self.on_trans_started)
		self.Handle("TransferProgress", self.on_trans_progress)
		self.Handle("TransferCompleted", self.on_trans_complete)
		self.Handle("ErrorOccurred", self.on_error)
		
	def __del__(self):
		dprint("deleting session")
		
	def on_cancelled(self):
		self.emit("cancelled")
		#self.DisconnectAll()
		
	def on_disconnected(self):
		self.emit("disconnected")
		#self.DisconnectAll()
		
	def on_trans_started(self, filename, path, size):
		self.emit("transfer-started", filename, path, size)
		
	def on_trans_progress(self, bytes):
		self.emit("transfer-progress", bytes)
		
	def on_trans_complete(self):
		self.emit("transfer-completed")
		
	def on_error(self, name, msg):
		self.emit("error-occurred", name, msg)
		#self.DisconnectAll()



# Copyright (C) 2009 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from blueman.plugins.MechanismPlugin import MechanismPlugin
import os
import subprocess
from gi.repository import GObject
from blueman.main.NetConf import NetConf, DnsMasqHandler, DhcpdHandler

class Network(MechanismPlugin):
	def on_load(self):
		self.add_dbus_method(self.SetGN, in_signature="b", out_signature="", sender_keyword="caller")
		self.add_dbus_method(self.NetworkSetup, in_signature="sbs", out_signature="", sender_keyword="caller")
		self.add_dbus_method(self.DhcpClient, in_signature="s", out_signature="s", sender_keyword="caller", async_callbacks=("ok", "err"))
	
		self.add_dbus_method(self.EnableNetwork, in_signature="ayays", out_signature="", sender_keyword="caller", byte_arrays=True)
		self.add_dbus_method(self.DisableNetwork, in_signature="", out_signature="", sender_keyword="caller")
		self.add_dbus_method(self.ReloadNetwork, in_signature="", out_signature="", sender_keyword="caller")
	
	def DhcpClient(self, net_interface, caller, ok, err):
		self.timer.stop()

		self.confirm_authorization(caller, "org.blueman.dhcp.client")
		
		from blueman.main.DhcpClient import DhcpClient
		def dh_error(dh, message, ok, err):
			err(message)
			self.timer.resume()
		
		def dh_connected(dh, ip, ok, err):
			ok(ip)
			self.timer.resume()
		
		dh = DhcpClient(net_interface)
		dh.connect("error-occurred", dh_error, ok, err)
		dh.connect("connected", dh_connected, ok, err)		
		try:
			dh.Connect()
		except Exception, e:
			err(e)
		
	def SetGN(self, enabled, caller):
		self.timer.reset()
		if enabled:	
			p = subprocess.Popen(["/usr/sbin/avahi-autoipd", "-D", "pan0"], env=os.environ, bufsize=128)
		else:
			p = subprocess.Popen(["/usr/sbin/avahi-autoipd", "-k", "pan0"], bufsize=128)
		
		#reap the child
		GObject.child_watch_add(p.pid, lambda pid, cond: 0)
	
	def EnableNetwork(self, ip_address, netmask, dhcp_handler, caller):
		nc = NetConf.get_default()
		nc.set_ipv4(ip_address, netmask)
		eval("nc.set_dhcp_handler(%s)" % dhcp_handler)
		nc.apply_settings()
		
	def ReloadNetwork(self, caller):
		nc = NetConf.get_default()
		nc.apply_settings()
	
	def DisableNetwork(self, caller):
		nc = NetConf.get_default()
		nc.remove_settings()
		nc.set_ipv4(None, None)
		nc.store()
	
	def NetworkSetup(self, ip_address, allow_nat, server_type, caller):
		self.timer.reset()
		dprint(ip_address, allow_nat, server_type)
		if ip_address == "reload":
			info = netstatus()
			nc = None
			if info["ip"] != "0" and not nc_is_running():
				if info["type"] == "dnsmasq":
					nc = NetConfDnsMasq(None)
				elif info["type"] == "dhcpd":
					nc = NetConfDhcpd(None)
				
				if nc:
					nc.reload_settings()
			
			return
		
		self.confirm_authorization(caller, "org.blueman.network.setup")
		if ip_address == "0":
			info = netstatus()
			nc = None
			try:
				if info["type"] == "dnsmasq":
					nc = NetConfDnsMasq(None)
				elif info["type"] == "dhcpd":
					nc = NetConfDhcpd(None)
			except:
				#fallback
				nc = NetConf(None)
			
			nc.uninstall()

		else:
			if ip_chk(ip_address):
				nc = None
				if server_type == "dnsmasq":
					nc = NetConfDnsMasq(ip_address, allow_nat)
				elif server_type == "dhcpd":
					nc = NetConfDhcpd(ip_address, allow_nat)
				if nc:
					nc.install()

			else:
				return dbus.DBusException("IP Invalid")




from blueman.bluez.BlueZInterface import BlueZInterface
from blueman.plugins.ManagerPlugin import ManagerPlugin
from gi.repository import Gtk

from blueman.Sdp import *
from blueman.Functions import *
from blueman.main.SignalTracker import SignalTracker
from blueman.gui.manager.ManagerProgressbar import ManagerProgressbar
from blueman.main.Config import Config
from blueman.main.AppletService import AppletService
from blueman.gui.MessageArea import MessageArea

from blueman.Lib import rfcomm_list


def get_x_icon(icon_name, size):
    ic = get_icon(icon_name, size)
    x = get_icon("blueman-x", size)
    pixbuf = composite_icon(ic, [(x, 0, 0, 255)])

    return pixbuf


class Services(ManagerPlugin):
    connectable_uuids = [HID_SVCLASS_ID, AUDIO_SOURCE_SVCLASS_ID, AUDIO_SINK_SVCLASS_ID, HEADSET_SVCLASS_ID, HANDSFREE_SVCLASS_ID]

    def on_request_menu_items(self, manager_menu, device):
        if BlueZInterface.get_interface_version()[0] < 5:
            return self.on_request_menu_items_bluez4(manager_menu, device)

        items = []
        uuids = device.UUIDs
        appl = AppletService()

        uuids16 = [uuid128_to_uuid16(uuid) for uuid in uuids]

        if set(uuids16) & set(self.connectable_uuids):
            # FIXME: This should only be done once!
            manager_menu.Signals.Handle("bluez", device, manager_menu.service_property_changed, "PropertyChanged")

            if device.get_properties()['Connected']:
                # FIXME: More generic icon
                item = create_menuitem(_("Disconnect"), get_x_icon("mouse", 16))
                manager_menu.Signals.Handle("gobject", item, "activate", manager_menu.on_disconnect, device)
                items.append((item, 100))
            else:
                # FIXME: More generic icon
                item = create_menuitem(_("Connect"), get_icon("mouse", 16))
                manager_menu.Signals.Handle("gobject", item, "activate", manager_menu.on_connect, device)
                items.append((item, 1))

            item.show()

        for name, service in device.Services.items():
            if name == "network":
                manager_menu.Signals.Handle("bluez", service, manager_menu.service_property_changed, "PropertyChanged")
                sprops = service.get_properties()

                if not sprops["Connected"]:
                    for uuid in uuids:
                        uuid16 = uuid128_to_uuid16(uuid)
                        if uuid16 == NAP_SVCLASS_ID or uuid16 == GN_SVCLASS_ID:
                            label = _("Group Network") if uuid16 == GN_SVCLASS_ID else _("Network Access Point")
                            item = create_menuitem(label, get_icon("network-wireless", 16))
                            manager_menu.Signals.Handle("gobject", item, "activate", manager_menu.on_connect,
                                                        device, name, uuid)
                            item.show()
                            items.append((item, 80))
                else:
                    item = create_menuitem(_("Network"), get_x_icon("network-wireless", 16))
                    manager_menu.Signals.Handle("gobject", item, "activate", manager_menu.on_disconnect, device, name)
                    item.show()
                    items.append((item, 101))

                    if "DhcpClient" in appl.QueryPlugins():
                        def renew(x):
                            appl.DhcpClient(sprops["Interface"])

                        item = create_menuitem(_("Renew IP Address"), get_icon("gtk-refresh", 16))
                        manager_menu.Signals.Handle("gobject", item, "activate", renew)
                        item.show()
                        items.append((item, 201))

        return items

    def on_request_menu_items_bluez4(self, manager_menu, device):
        # BlueZ 4 only

        items = []
        uuids = device.UUIDs
        appl = AppletService()

        for name, service in device.Services.items():
            if name == "serial":
                ports_list = rfcomm_list()

                def flt(dev):
                    if dev["dst"] == device.Address and dev["state"] == "connected":
                        return dev["channel"]

                active_ports = map(flt, ports_list)


                def get_port_id(channel):
                    for dev in ports_list:
                        if dev["dst"] == device.Address and dev["state"] == "connected" and dev["channel"] == channel:
                            return dev["id"]

                serial_items = []

                num_ports = 0
                has_dun = False
                try:
                    for port_name, channel, uuid in sdp_get_cached_rfcomm(device.Address):

                        if SERIAL_PORT_SVCLASS_ID in uuid:
                            if name is not None:
                                if channel in active_ports:
                                    item = create_menuitem(port_name, get_x_icon("blueman-serial", 16))
                                    manager_menu.Signals.Handle("gobject",
                                                                item,
                                                                "activate",
                                                                manager_menu.on_disconnect,
                                                                device, name,
                                                                "/dev/rfcomm%d" % get_port_id(channel))
                                    item.show()
                                    items.append((item, 150))
                                else:
                                    item = create_menuitem(port_name, get_icon("blueman-serial", 16))
                                    manager_menu.Signals.Handle("gobject",
                                                                item,
                                                                "activate",
                                                                manager_menu.on_connect,
                                                                device, name,
                                                                channel)
                                    item.show()
                                    serial_items.append(item)


                        elif DIALUP_NET_SVCLASS_ID in uuid:
                            if name is not None:
                                if channel in active_ports:
                                    item = create_menuitem(port_name, get_x_icon("modem", 16))
                                    manager_menu.Signals.Handle("gobject",
                                                                item,
                                                                "activate",
                                                                manager_menu.on_disconnect,
                                                                device,
                                                                name,
                                                                "/dev/rfcomm%d" % get_port_id(channel))
                                    item.show()
                                    items.append((item, 150))
                                else:
                                    item = create_menuitem(port_name, get_icon("modem", 16))
                                    manager_menu.Signals.Handle("gobject",
                                                                item,
                                                                "activate",
                                                                manager_menu.on_connect,
                                                                device,
                                                                name,
                                                                channel)
                                    item.show()
                                    serial_items.append(item)
                                    has_dun = True

                except KeyError:
                    for uuid in uuids:
                        uuid16 = uuid128_to_uuid16(uuid)
                        if uuid16 == DIALUP_NET_SVCLASS_ID:
                            item = create_menuitem(_("Dialup Service"), get_icon("modem", 16))
                            manager_menu.Signals.Handle("gobject",
                                                        item,
                                                        "activate",
                                                        manager_menu.on_connect,
                                                        device,
                                                        name,
                                                        uuid)
                            item.show()
                            serial_items.append(item)
                            has_dun = True

                        if uuid16 == SERIAL_PORT_SVCLASS_ID:
                            item = create_menuitem(_("Serial Service"),
                                                   get_icon("blueman-serial",
                                                            16))
                            manager_menu.Signals.Handle("gobject",
                                                        item,
                                                        "activate",
                                                        manager_menu.on_connect,
                                                        device, name, uuid)
                            item.show()
                            serial_items.append(item)

                    for dev in ports_list:
                        if dev["dst"] == device.Address:
                            if dev["state"] == "connected":
                                devname = _("Serial Port %s") % "rfcomm%d" % dev["id"]

                                item = create_menuitem(devname, get_x_icon("modem", 16))
                                manager_menu.Signals.Handle("gobject",
                                                            item,
                                                            "activate",
                                                            manager_menu.on_disconnect,
                                                            device, name,
                                                            "/dev/rfcomm%d" % dev["id"])
                                items.append((item, 120))
                                item.show()

                def open_settings(i, device):
                    from blueman.gui.GsmSettings import GsmSettings

                    d = GsmSettings(device.Address)
                    d.run()
                    d.destroy()

                if has_dun and "PPPSupport" in appl.QueryPlugins():
                    item = Gtk.SeparatorMenuItem()
                    item.show()
                    serial_items.append(item)

                    item = create_menuitem(_("Dialup Settings"),
                                           get_icon("gtk-preferences", 16))
                    serial_items.append(item)
                    item.show()
                    manager_menu.Signals.Handle("gobject", item,
                                                "activate", open_settings,
                                                device)

                if len(serial_items) > 1:
                    sub = Gtk.Menu()
                    sub.show()

                    item = create_menuitem(_("Serial Ports"), get_icon("modem", 16))
                    item.set_submenu(sub)
                    item.show()
                    items.append((item, 90))

                    for item in serial_items:
                        sub.append(item)

                else:
                    for item in serial_items:
                        items.append((item, 80))


            elif name == "network":
                manager_menu.Signals.Handle("bluez",
                                            service,
                                            manager_menu.service_property_changed,
                                            "PropertyChanged")

                sprops = service.get_properties()

                if not sprops["Connected"]:

                    for uuid in uuids:
                        uuid16 = uuid128_to_uuid16(uuid)
                        if uuid16 == GN_SVCLASS_ID:
                            item = create_menuitem(_("Group Network"),
                                                   get_icon("network-wireless", 16))
                            manager_menu.Signals.Handle("gobject",
                                                        item,
                                                        "activate",
                                                        manager_menu.on_connect,
                                                        device, name, uuid)
                            item.show()
                            items.append((item, 80))

                        if uuid16 == NAP_SVCLASS_ID:
                            item = create_menuitem(_("Network Access Point"),
                                                   get_icon("network-wireless", 16))
                            manager_menu.Signals.Handle("gobject",
                                                        item,
                                                        "activate",
                                                        manager_menu.on_connect,
                                                        device, name, uuid)
                            item.show()
                            items.append((item, 81))


                else:
                    item = create_menuitem(_("Network"), get_x_icon("network-wireless", 16))
                    manager_menu.Signals.Handle("gobject", item,
                                                "activate",
                                                manager_menu.on_disconnect,
                                                device, name)
                    item.show()
                    items.append((item, 101))

                    if "DhcpClient" in appl.QueryPlugins():
                        def renew(x):
                            appl.DhcpClient(sprops["Interface"])

                        item = create_menuitem(_("Renew IP Address"), get_icon("gtk-refresh", 16))
                        manager_menu.Signals.Handle("gobject", item, "activate", renew)
                        item.show()
                        items.append((item, 201))

            elif name == "input":
                manager_menu.Signals.Handle("bluez",
                                            service,
                                            manager_menu.service_property_changed,
                                            "PropertyChanged")
                sprops = service.get_properties()
                if sprops["Connected"]:
                    item = create_menuitem(_("Input Service"), get_x_icon("mouse", 16))
                    manager_menu.Signals.Handle("gobject",
                                                item,
                                                "activate",
                                                manager_menu.on_disconnect,
                                                device, name)
                    items.append((item, 100))

                else:
                    item = create_menuitem(_("Input Service"), get_icon("mouse", 16))
                    manager_menu.Signals.Handle("gobject",
                                                item,
                                                "activate",
                                                manager_menu.on_connect,
                                                device, name)
                    items.append((item, 1))

                item.show()


            elif name == "headset":
                sprops = service.get_properties()

                if sprops["Connected"]:
                    item = create_menuitem(_("Headset Service"),
                                           get_icon("blueman-handsfree", 16))
                    manager_menu.Signals.Handle("gobject",
                                                item, "activate",
                                                manager_menu.on_disconnect,
                                                device, name)
                    items.append((item, 110))
                else:
                    item = create_menuitem(_("Headset Service"),
                                           get_icon("blueman-handsfree", 16))
                    manager_menu.Signals.Handle("gobject", item,
                                                "activate",
                                                manager_menu.on_connect,
                                                device, name)
                    items.append((item, 10))

                item.show()



            elif name == "audiosink":
                sprops = service.get_properties()

                if sprops["Connected"]:
                    item = create_menuitem(_("Audio Sink"), get_icon("blueman-headset", 16))
                    manager_menu.Signals.Handle("gobject",
                                                item,
                                                "activate",
                                                manager_menu.on_disconnect,
                                                device, name)
                    items.append((item, 120))
                else:
                    item = create_menuitem(_("Audio Sink"),
                                           get_icon("blueman-headset", 16))
                    item.props.tooltip_text = _("Allows to send audio to remote device")
                    manager_menu.Signals.Handle("gobject",
                                                item,
                                                "activate",
                                                manager_menu.on_connect,
                                                device, name)
                    items.append((item, 20))

                item.show()


            elif name == "audiosource":
                sprops = service.get_properties()

                if not sprops["State"] == "disconnected":
                    item = create_menuitem(_("Audio Source"),
                                           get_icon("blueman-headset", 16))
                    manager_menu.Signals.Handle("gobject",
                                                item, "activate",
                                                manager_menu.on_disconnect,
                                                device, name)
                    items.append((item, 121))
                else:
                    item = create_menuitem(_("Audio Source"),
                                           get_icon("blueman-headset", 16))
                    item.props.tooltip_text = _("Allows to receive audio from remote device")
                    manager_menu.Signals.Handle("gobject",
                                                item, "activate",
                                                manager_menu.on_connect,
                                                device, name)
                    items.append((item, 21))
                item.show()

        return items




# Copyright (C) 2008 Valmantas Paliksa <walmis at balticum-tv dot lt>
# Copyright (C) 2008 Tadas Dailyda <tadas at dailyda dot com>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 
from blueman.Functions import *
from blueman.plugins.AppletPlugin import AppletPlugin
from blueman.gui.CommonUi import show_about_dialog
from blueman.gui.applet.PluginDialog import PluginDialog

from gi.repository import GObject
from gi.repository import Gtk

class StandardItems(AppletPlugin):
	__depends__ = ["StatusIcon", "Menu"]
	__unloadable__ = False
	__description__ = _("Adds standard menu items to the status icon menu")
	__author__ = "walmis"
	
	def on_load(self, applet):
		self.Applet = applet
		
		applet.Plugins.Menu.Register(self, Gtk.SeparatorMenuItem(), 21)
		
		self.new_dev = create_menuitem(_("_Setup New Device")+"...", get_icon("gtk-new", 16))
		self.new_dev.connect("activate", self.on_setup_new)
		
		self.Applet.Plugins.Menu.Register(self, self.new_dev, 30)
		
		self.Applet.Plugins.Menu.Register(self, Gtk.SeparatorMenuItem(), 31)
		
		self.send = create_menuitem(_("Send _Files to Device")+"...", get_icon("blueman-send-file", 16))
		self.send.connect("activate", self.on_send)

		self.Applet.Plugins.Menu.Register(self, self.send, 40)

		self.browse = create_menuitem(_("_Browse Files on Device")+"...", get_icon("gtk-open", 16))
		self.browse.connect("activate", self.on_browse)
		
		self.Applet.Plugins.Menu.Register(self, self.browse, 50)
		
		self.Applet.Plugins.Menu.Register(self, Gtk.SeparatorMenuItem(), 51)
		

		self.devices = Gtk.MenuItem.new_with_mnemonic(_("_Devices")+"...")
		self.devices.connect("activate", self.on_devices)
		
		self.Applet.Plugins.Menu.Register(self, self.devices, 60)
		
		self.adapters = create_menuitem(_("Adap_ters")+"...", get_icon("blueman-device", 16))
		self.adapters.connect("activate", self.on_adapters)
		
		self.Applet.Plugins.Menu.Register(self, self.adapters, 70)
		
		self.services = create_menuitem(_("_Local Services")+"...", get_icon("gtk-preferences", 16))
		self.services.connect("activate", self.on_local_services)
		
		self.Applet.Plugins.Menu.Register(self, self.services, 80)
		
		self.Applet.Plugins.Menu.Register(self, Gtk.SeparatorMenuItem(), 81)
		
		about = Gtk.ImageMenuItem.new_from_stock(Gtk.STOCK_ABOUT, None)
		self.Applet.Plugins.Menu.Register(self, about, 90)
		
		self.plugins = create_menuitem(_("_Plugins"), get_icon("blueman-plugin", 16))
		self.plugins.connect("activate", self.on_plugins)
		
		self.Applet.Plugins.Menu.Register(self, self.plugins, 85)
		
		about.connect("activate", self.on_about)
		
		def on_activate(status_icon):
			self.on_devices(None)
			
			
		
		self.Applet.Plugins.StatusIcon.connect("activate", on_activate)
		
	def change_sensitivity(self, sensitive):
		try:
			power = self.Applet.Plugins.PowerManager.GetBluetoothStatus()
		except:
			power = True
			
		sensitive = sensitive and self.Applet.Manager and power
		self.new_dev.props.sensitive = sensitive
		self.send.props.sensitive = sensitive
		self.browse.props.sensitive = sensitive
		self.devices.props.sensitive = sensitive
		self.adapters.props.sensitive = sensitive
		
	def on_manager_state_changed(self, state):
		self.change_sensitivity(state)
		
	def on_power_state_changed(self, manager, state):
		self.change_sensitivity(state)
	
	
	def on_setup_new(self, menu_item):
		sn = startup_notification("Bluetooth Assistant", _("Starting Bluetooth Assistant"), bin_name="blueman-assistant", icon="blueman")
		spawn("blueman-assistant", sn=sn)
		
	def on_send(self, menu_item):
		sn = startup_notification("Blueman Sendto", _("Starting File Sender"), bin_name="blueman-sendto", icon="blueman")
		spawn("blueman-sendto", sn=sn)
		
	def on_browse(self, menu_item):
		sn = startup_notification("Blueman Browse", _("Starting File Browser"), bin_name="blueman-browse", icon="blueman")
		spawn("blueman-browse", sn=sn)
		
	def on_devices(self, menu_item):
		sn = startup_notification("Blueman Manager", _("Starting Device Manager"), bin_name="blueman-manger", icon="blueman")
		spawn("blueman-manager", sn=sn)
		
	def on_adapters(self, menu_item):
		sn = startup_notification("Blueman Adapters", _("Starting Adapter Preferences"), bin_name="blueman-adapters", icon="blueman")
		spawn("blueman-adapters", sn=sn)
		
	def on_local_services(self, menu_item):
		sn = startup_notification("Blueman Services", _("Starting Service Preferences"), bin_name="blueman-services", icon="blueman")
		spawn("blueman-services", sn=sn)
		
	def on_about(self, menu_item):
		about = show_about_dialog("Blueman "+_("applet"), run=False)
		
		button = Gtk.Button(_("Plugins"))
		button.set_image(Gtk.Image.new_from_icon_name("blueman-plugin", Gtk.IconSize.BUTTON))
		button.show()
		
		button.connect("clicked", self.on_plugins)
		
		about.action_area.pack_start(button, True, True, 0)
		about.action_area.reorder_child(button, 0)
		
		about.run()
		about.destroy()
		
	def on_plugins(self, button):
		dialog = PluginDialog(self.Applet)
		dialog.run()
		dialog.destroy()
		
		
				



from gi.repository import Gtk
from gi.repository import GObject
from blueman.gui.DeviceSelectorWidget import DeviceSelectorWidget


class DeviceSelectorDialog(Gtk.Dialog):
    def __init__(self, title=_("Select Device"), parent=None, discover=True):

        Gtk.Dialog.__init__(self, title, parent, 0, (Gtk.STOCK_CANCEL, Gtk.ResponseType.REJECT,
                                                     Gtk.STOCK_OK, Gtk.ResponseType.ACCEPT))

        self.props.resizable = False
        self.props.icon_name = "blueman"
        self.selector = DeviceSelectorWidget()
        self.selector.show()

        #self.selector.destroy()
        #self.selector = None

        align = Gtk.Alignment.new(0.5, 0.5, 1.0, 1.0)
        align.add(self.selector)

        align.set_padding(6, 6, 6, 6)
        align.show()
        self.vbox.pack_start(align, True, True, 0)


        #(adapter, device)
        self.selection = None

        self.selector.List.connect("device-selected", self.on_device_selected)
        self.selector.List.connect("adapter-changed", self.on_adapter_changed)
        if discover:
            self.selector.List.DiscoverDevices()

        self.selector.List.connect("row-activated", self.on_row_activated)

    def on_row_activated(self, treeview, path, view_column, *args):
        self.response(Gtk.ResponseType.ACCEPT)

    def on_adapter_changed(self, devlist, adapter):
        self.selection = None

    def on_device_selected(self, devlist, device, iter):
        self.selection = (devlist.Adapter.get_object_path(), device)

    def GetSelection(self):
        if self.selection:
            return (self.selection[0], self.selection[1].Copy())
        else:
            return None



from blueman.Functions import *
import pickle
import base64
from blueman.main.Config import Config
from blueman.Sdp import parse_sdp_xml, sdp_save
from blueman.plugins.AppletPlugin import AppletPlugin
from blueman.main.applet.BluezAgent import AdapterAgent

from blueman.bluez.Device import Device as BluezDevice
from blueman.main.Device import Device
from blueman.main.applet.BluezAgent import TempAgent
from blueman.bluez.Adapter import Adapter
from blueman.bluez.Serial import Serial

from gi.repository import GObject
from gi.repository import Gtk
import dbus


class DBusService(AppletPlugin):
    __depends__ = ["StatusIcon"]
    __unloadable__ = False
    __description__ = _("Provides DBus API for other Blueman components")
    __author__ = "Walmis"

    def on_load(self, applet):
        self.Applet = applet

        AppletPlugin.add_method(self.on_rfcomm_connected)
        AppletPlugin.add_method(self.on_rfcomm_disconnect)
        AppletPlugin.add_method(self.rfcomm_connect_handler)
        AppletPlugin.add_method(self.service_connect_handler)
        AppletPlugin.add_method(self.on_device_disconnect)

        self.add_dbus_method(self.ServiceProxy, in_signature="sosas", async_callbacks=("ok", "err"))
        self.add_dbus_method(self.CreateDevice, in_signature="ssbu", async_callbacks=("_ok", "err"))
        self.add_dbus_method(self.CancelDeviceCreation, in_signature="ss", async_callbacks=("ok", "err"))
        self.add_dbus_method(self.RfcommConnect, in_signature="ss", out_signature="s", async_callbacks=("ok", "err"))
        self.add_dbus_method(self.RfcommDisconnect, in_signature="ss", out_signature="")
        self.add_dbus_method(self.RefreshServices, in_signature="s", out_signature="", async_callbacks=("ok", "err"))

        self.add_dbus_method(self.QueryPlugins, in_signature="", out_signature="as")
        self.add_dbus_method(self.QueryAvailablePlugins, in_signature="", out_signature="as")
        self.add_dbus_method(self.SetPluginConfig, in_signature="sb", out_signature="")
        self.add_dbus_method(self.DisconnectDevice, in_signature="o", out_signature="", async_callbacks=("ok", "err"))

    def RefreshServices(self, path, ok, err):
        # BlueZ 4 only!
        device = Device(path)

        def reply(svcs):
            try:
                records = parse_sdp_xml(svcs)
                sdp_save(device.Address, records)
            except:
                pass
            ok()

        device.get_interface().DiscoverServices("", reply_handler=reply, error_handler=err)

    def QueryPlugins(self):
        return self.Applet.Plugins.GetLoaded()

    def DisconnectDevice(self, obj_path, ok, err):
        dev = Device(obj_path)

        self.Applet.Plugins.Run("on_device_disconnect", dev)

        def on_timeout():
            dev.Disconnect(reply_handler=ok, error_handler=err)

        GObject.timeout_add(1000, on_timeout)

    def on_device_disconnect(self, device):
        pass

    def QueryAvailablePlugins(self):
        return self.Applet.Plugins.GetClasses()

    def SetPluginConfig(self, plugin, value):
        self.Applet.Plugins.SetConfig(plugin, value)

    def ConnectHelper(self, interface, object_path, _method, args, ok, err):
        bus = dbus.SystemBus()
        service = bus.get_object("org.bluez", object_path)
        method = service.get_dbus_method(_method, interface)

        method(reply_handler=ok, error_handler=err, *args)


    def ServiceProxy(self, interface, object_path, _method, args, ok, err):

        if _method == "Connect":
            dev = Device(object_path)
            try:
                self.Applet.Plugins.RecentConns.notify(dev, interface, args)
            except KeyError:
                dprint("RecentConns plugin is unavailable")

        self.handled = False

        def cb(inst, ret):
            if ret == True:
                self.handled = True
                #stop further execution
                raise StopException

        self.Applet.Plugins.RunEx("service_connect_handler", cb, interface, object_path, _method, args, ok, err)

        if not self.handled:
            self.ConnectHelper(interface, object_path, _method, args, ok, err)

        del self.handled

    def service_connect_handler(self, interface, object_path, _method, args, ok, err):
        pass

    def CreateDevice(self, adapter_path, address, pair, time, _ok, err):
        # BlueZ 4 only!
        def ok(device):
            path = device.get_object_path()
            _ok(path)
            self.RefreshServices(path, (lambda *args: None), (lambda *args: None))

        if self.Applet.Manager:
            adapter = Adapter(adapter_path)

            if pair:
                agent_path = "/org/blueman/agent/temp/" + address.replace(":", "")
                agent = TempAgent(self.Applet.Plugins.StatusIcon, agent_path, time)
                adapter.create_paired_device(address, agent_path, "DisplayYesNo", error_handler=err,
                                             reply_handler=ok, timeout=120)

            else:
                adapter.create_device(address, error_handler=err, reply_handler=ok, timeout=120)

        else:
            err()

    def CancelDeviceCreation(self, adapter_path, address, ok, err):
        # BlueZ 4 only!
        if self.Applet.Manager:
            adapter = Adapter(adapter_path)

            adapter.get_interface().CancelDeviceCreation(address, error_handler=err, reply_handler=ok)

        else:
            err()

    def RfcommConnect(self, device, uuid, ok, err):
        def reply(rfcomm):
            self.Applet.Plugins.Run("on_rfcomm_connected", dev, rfcomm, uuid)
            ok(rfcomm)

        dev = Device(device)
        try:
            self.Applet.Plugins.RecentConns.notify(dev.Copy(), Serial().get_interface_name(), [uuid])
        except KeyError:
            pass

        rets = self.Applet.Plugins.Run("rfcomm_connect_handler", dev, uuid, reply, err)
        if True in rets:
            pass
        else:
            dprint("No handler registered")
            err(dbus.DBusException(
                "Service not supported\nPossibly the plugin that handles this service is not loaded"))


    def rfcomm_connect_handler(self, device, uuid, reply_handler, error_handler):
        return False


    def RfcommDisconnect(self, device, rfdevice):
        dev = Device(BluezDevice(device))
        dev.Services["serial"].Disconnect(rfdevice)

        self.Applet.Plugins.Run("on_rfcomm_disconnect", rfdevice)

        dprint("Disonnecting rfcomm device")

    def on_rfcomm_connected(self, device, port, uuid):
        pass

    def on_rfcomm_disconnect(self, port):
        pass



# Copyright (C) 2009 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# 

from blueman.plugins.MechanismPlugin import MechanismPlugin
import os

class RfKill(MechanismPlugin):
	def on_load(self):
		self.add_dbus_method(self.SetRfkillState, in_signature="b", out_signature="")
		self.add_dbus_method(self.DevRfkillChmod, in_signature="", out_signature="")
		
	def SetRfkillState(self, state):		
		from blueman.main.KillSwitchNG import KillSwitchNG
		k = KillSwitchNG()
		k.SetGlobalState(state)

	def DevRfkillChmod(self):
		try:
			os.chmod("/dev/rfkill", 0655)
		except:
			pass



from blueman.Functions import dprint
import tty
import termios
import os
import subprocess
from gi.repository import GObject
import errno
import re

pppd_errors = {
    1: """An immediately fatal error of some kind  occurred, such as an essential system call failing, or running out of virtual memory.""",
    2: """An  error  was detected in processing the options given, such as two mutually exclusive options being used.""",
    3: """Pppd is not setuid-root and the invoking user is not root.""",
    4: """The kernel does not support PPP, for example, the  PPP kernel driver is not included or cannot be loaded.""",
    5: """Pppd terminated because it was sent a SIGINT, SIGTERM or SIGHUP signal.""",
    6: """The serial port could not be locked.""", 7: """The serial port could not be opened.""",
    8: """The connect script failed (returned a non-zero exit status).""",
    9: """The command specified as the argument to the  pty  option  could not be run.""",
    10: """The PPP negotiation failed, that is, it didn't reach the point where at least one network protocol (e.g. IP) was running.""",
    11: """The peer system failed (or refused) to authenticate itself.""",
    12: """The link was established successfully and terminated because  it was idle.""",
    13: """The link was established successfully and terminated because the connect time limit was reached.""",
    14: """Callback was negotiated  and  an  incoming  call  should  arrive shortly.""",
    15: """The link was terminated because the peer is not responding to echo requests.""",
    16: """The link was terminated by the modem hanging up.""",
    17: """The PPP negotiation failed because serial loopback was detected.""",
    18: """The init script failed (returned a non-zero exit status).""",
    19: """We failed to authenticate ourselves to the peer."""
}


class PPPException(Exception):
    pass


class PPPConnection(GObject.GObject):
    __gsignals__ = {  # arg: interface name eg. ppp0
        'connected': (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT,)),
        'error-occurred': (GObject.SignalFlags.NO_HOOKS, None, (GObject.TYPE_PYOBJECT,))
    }

    def __init__(self, port, number="*99#", apn="", user="", pwd=""):
        GObject.GObject.__init__(self)

        self.apn = apn
        self.number = number
        self.user = user
        self.pwd = pwd
        self.port = port
        self.interface = None

        self.pppd = None
        self.file = None

        self.commands = [
            ("ATZ E0 V1 X4 &C1 +FCLASS=0", self.simple_callback),
            ("ATE0", self.simple_callback),
            ("AT+GCAP", self.simple_callback),
            (
                "ATD%s" % self.number,
                self.connect_callback,
                ["CONNECT", "NO CARRIER", "BUSY", "NO ANSWER", "NO DIALTONE", "OK", "ERROR"]
            )
        ]
        if self.apn != "":
            self.commands.insert(-1, ('AT+CGDCONT=1,"IP","%s"' % self.apn, self.simple_callback))

    def cleanup(self):
        os.close(self.file)
        self.file = None

    def simple_callback(self, response):
        pass

    def connect_callback(self, response):
        if "CONNECT" in response:
            dprint("Starting pppd")
            self.pppd = subprocess.Popen(
                ["/usr/sbin/pppd", "%s" % self.port, "115200", "defaultroute", "updetach", "usepeerdns"], bufsize=1,
                stdout=subprocess.PIPE)
            GObject.io_add_watch(self.pppd.stdout, GObject.IO_IN | GObject.IO_ERR | GObject.IO_HUP, self.on_pppd_stdout)
            GObject.timeout_add(1000, self.check_pppd)

            self.cleanup()
        else:
            self.cleanup()
            raise PPPException("Bad modem response %s, expected CONNECT" % response[0])

    def __cmd_response_cb(self, response, exception, item_id):
        if exception:
            self.emit("error-occurred", str(exception))
        else:
            try:
                self.commands[item_id][1](response)
            except PPPException as e:
                self.emit("error-occurred", str(e))
                return

            self.send_commands(item_id + 1)

    def send_commands(self, id=0):
        try:
            item = self.commands[id]
        except IndexError:
            return

        if len(item) == 3:
            (command, callback, terminators) = item
        else:
            (command, callback) = item
            terminators = ["OK", "ERROR"]

        self.send_command(command)
        self.wait_for_reply(self.__cmd_response_cb, terminators, id)

    def Connect(self):

        self.file = os.open(self.port, os.O_RDWR | os.O_EXCL | os.O_NONBLOCK | os.O_NOCTTY)

        tty.setraw(self.file)

        attrs = termios.tcgetattr(self.file)

        attrs[0] &= ~(termios.IGNCR | termios.ICRNL | termios.IUCLC | termios.INPCK | termios.IXON | termios.IXANY |
                      termios.IGNPAR)
        attrs[1] &= ~(termios.OPOST | termios.OLCUC | termios.OCRNL | termios.ONLCR | termios.ONLRET)
        attrs[3] &= ~(termios.ICANON | termios.XCASE | termios.ECHO | termios.ECHOE | termios.ECHONL)
        attrs[3] &= ~(termios.ECHO | termios.ECHOE)
        attrs[6][termios.VMIN] = 1
        attrs[6][termios.VTIME] = 0
        attrs[6][termios.VEOF] = 1

        attrs[2] &= ~(termios.CBAUD | termios.CSIZE | termios.CSTOPB | termios.CLOCAL | termios.PARENB)
        attrs[2] |= (termios.B9600 | termios.CS8 | termios.CREAD | termios.PARENB)

        termios.tcsetattr(self.file, termios.TCSANOW, attrs)

        termios.tcflush(self.file, termios.TCIOFLUSH)

        self.send_commands()

    def on_pppd_stdout(self, source, cond):
        if cond & GObject.IO_ERR or cond & GObject.IO_HUP:
            return False

        line = source.readline()
        m = re.match("Using interface (ppp[0-9]*)", line)
        if m:
            self.interface = m.groups(1)[0]

        print(line)

        return True

    def check_pppd(self):
        status = self.pppd.poll()
        if status is not None:
            if status == 0:
                self.emit("connected", self.interface)
            else:
                try:
                    msg = "pppd exited: " + pppd_errors[int(status)]
                except KeyError:
                    msg = "pppd exited with unknown error"

                self.emit("error-occurred", msg)

            print("pppd exited with status %d" % status)
            return False
        return True

    def send_command(self, command):
        dprint("-->", command)
        os.write(self.file, "%s\r\n" % command)
        termios.tcdrain(self.file)

    def on_data_ready(self, source, condition, terminators, on_done):
        if condition & GObject.IO_ERR or condition & GObject.IO_HUP:
            on_done(None, PPPException("Socket error"))
            self.cleanup()
            return False
        try:
            self.buffer += os.read(self.file, 1)
        except OSError as e:
            if e.errno == errno.EAGAIN:
                dprint("Got EAGAIN")
                return True
            else:
                on_done(None, PPPException("Socket error"))
                dprint(e)
                self.cleanup()
                return False

        lines = self.buffer.split("\r\n")
        found = False
        for l in lines:
            if l == "":
                pass
            else:
                for t in terminators:
                    if t in l:
                        found = True

        if found:
            lines = filter(lambda x: x != "", lines)
            lines = map(lambda x: x.strip("\r\n"), lines)
            dprint("<-- ", lines)

            on_done(lines, None)
            return False

        return True

    def wait_for_reply(self, callback, terminators=["OK", "ERROR"], *user_data):
        def on_timeout():
            GObject.source_remove(self.io_watch)
            callback(None, PPPException("Modem initialization timed out"), *user_data)
            self.cleanup()
            return False

        def on_done(ret, exception):
            GObject.source_remove(self.timeout)
            callback(ret, exception, *user_data)


        self.buffer = ""
        self.term_found = False

        self.io_watch = GObject.io_add_watch(self.file, GObject.IO_IN | GObject.IO_ERR | GObject.IO_HUP, self.on_data_ready,
                                          terminators, on_done)
        self.timeout = GObject.timeout_add(15000, on_timeout)



# Copyright (C) 2010 Valmantas Paliksa <walmis at balticum-tv dot lt>
#
# Licensed under the GNU General Public License Version 3
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
from blueman.main.Config import Config
from blueman.plugins.BasePlugin import BasePlugin

class ConfigurablePlugin(BasePlugin):
	
	__options__ = {}
	
	@classmethod
	def is_configurable(cls):
		res = map(lambda x: (len(x) > 2), cls.__options__.values())
		return True in res
		
	def get_option(self, name):
		if not name in self.__class__.__options__:
			raise KeyError, "No such option"
		return getattr(self.__config.props, name)
		
	def set_option(self, name, value):
		if not name in self.__class__.__options__:
			raise KeyError, "No such option"
		opt = self.__class__.__options__[name]
		if type(value) == opt["type"]:
			setattr(self.__config.props, name, value)
			self.option_changed(name, value)
		else:
			raise TypeError, "Wrong type, must be %s" % repr(opt["type"])
			
	def option_changed(self, name, value):
		pass
			
	def __init__(self, parent):
		super(ConfigurablePlugin, self).__init__(parent)
			
		if self.__options__ != {}:
			self.__config = Config("plugins/" + self.__class__.__name__)
		 
			for k, v in self.__options__.iteritems():
				if getattr(self.__config.props, k) == None:
					setattr(self.__config.props, k, v["default"])
						
					
